/**************************************************************************************************
 *
 *	main.c
 *
 *	part of the layout program.
 *
 *
 *	Originally written by J. Barr 1991 (idea of J.Barr).
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	July 1991.
 *	email: a89@nikhef.nl
 *
 *************************************************************************************************/

# include "layout.h"

int Debug	= FALSE;		/* print statistics/jobs */
int number_of_nodes = 0;		/* keep track of total hosts - networks */
int WINDOWwidth  = 1000;		/* x-size of the window (default) */
int WINDOWheight  = 800;		/* y-size of the window (default) */
int DEFAULTwidth  = 150;		/* width for nodes with originally zero width */
int DEFAULTheight =  15;		/* height for nodes with originally zero height */
int boundary      = 100;		/* surrounding area in window, used for unlinked nodes */

struct hostentity *host		    = NULL;	/* name of the node structure */
struct hostentity *host_zero_links  = NULL;	/* structurename with nodes without links */
struct hostentity *host_one_link    = NULL;	/* structurename with nodes with only one links */
struct hostentity *host_more_to_one = NULL;	/* structurename with nodes with more links, but only to nodes with one link */

struct linkentity *link = NULL;			/* name of the link structure */


	/* Scans the input and builds-up a list with hostnames and their
	   coordinates and sizes . Copies this list into the output */
	/* gets everything done:
		layout of more_linked_nodes, placing of 1_link_node with another 1_link_node,
		placing of unlinked nodes
		printing of postscript etc */
   

void main(argc,argv)
int argc;
char *argv[];
{
	char line[MaxLine];			/* one line of input */
	int contr;				/* controls the scan-loop */
	char id[MaxEnt];			/* unique identifier + hostname */
	char name[MaxEnt];			/* hostname */
	int x_coord,y_coord;			/* initial x/y_coord of host */
	int width,height;			/* initial width/height of host */
	char type[MaxEnt];			/* type of link (connection) */
	char site_1[MaxEnt], site_2[MaxEnt];	/* first/second sitename of the connection */
	int cntrl;				/* counter used for scanning arguments */
	unsigned int it;			/* counts number of iterations to make */
	unsigned int iterations = 0;		/* the number of iterations */
	unsigned int con;
	struct hostentity *dummyhost1;		/* copy of structure host needed for printout */
	struct linkentity *dummylink1;		/* copy of structure link needed for printout */
	long UTC1, UTC2;			/* time used for determining layout */
	int printhost = FALSE;			/* print hoststructure or not */
	int printlink = FALSE;			/* print linkstructure or not */
	int rand_coord = FALSE;			/* flag to control randomly generated coordinates */
	int postscript = FALSE;			/* flag to control postscript output */
	int adjust_size = TRUE;			/* flag to control max. width/height for a node */
	float link_node_ratio;			/* ratio possible links for all nodes - the number of nodes */
	char interfacestring[MaxEnt];		/* string for scanning the line with connections between nodes */
	int length_int = 0;
	char dummystr[MaxEnt];
	float factor = 1;			/* factor to multiply the vector by (force determination) */


	strcpy(interfacestring,"interface=");

	if (argc > 14)
		usage(argv[0]);
	else if (argc == 1)
		iterations = 15;

	for (cntrl = 1 ; cntrl < argc ; cntrl++)
		switch(argv[cntrl][0])
		{
			case '-':
				switch(argv[cntrl][1])
				{
					case 'h':
						if ((cntrl + 1) < argc)
						{
							WINDOWheight = atoi(argv[cntrl + 1]);
							if (WINDOWheight < 500)
							{
								fprintf(stderr,"Too small: height resized to 500\n");
								WINDOWheight = 500;
							}
							cntrl ++;
						}
						else
						{
							fprintf(stderr,"You must enter the height of the window\n");
							usage(argv[0]);
						}
						break;
					case 'i':
						if ((cntrl + 1) < argc)
						{
							iterations = atoi(argv[cntrl + 1]);
							if (iterations < 1)
							{
								fprintf(stderr,"Number of iterations will be 1\n");
								iterations = 1;
							}
							cntrl ++;
						}
						else
						{
							fprintf(stderr,"You must enter the number of iterations to be made\n");
							usage(argv[0]);
						}
						break;
					case 'c':
						if ((cntrl + 1) < argc)
						{
							strcpy(interfacestring,argv[cntrl + 1]);
							length_int = strlen(interfacestring);
							cntrl ++;
						}
						else
						{
							fprintf(stderr,"You must enter the connectionname of the linked nodes\n");
							usage(argv[0]);
						}
						break;
					case 'f':
						if ((cntrl + 1) < argc)
						{
							factor = atof(argv[cntrl + 1]);
							if (factor <= 0)
							{
								fprintf(stderr,"You entered %d as factor, this is set to 1\n",factor);
								factor = 1;
							}
							else if (factor > 5)
								fprintf(stderr,"%d is a large factor\n",factor);
							cntrl ++;
						}
						else
						{
							fprintf(stderr,"You must enter the factor to multiply the forces with\n");
							usage(argv[0]);
						}
						break;
					case 'p':
						postscript = TRUE;
						break;
					case 'r':
						rand_coord = TRUE;
						break;
					case 's':
						adjust_size = FALSE;
						break;
					case 'w':
						if ((cntrl + 1) < argc)
						{
							WINDOWwidth = atoi(argv[cntrl + 1]);
							if (WINDOWwidth < 500)
							{
								fprintf(stderr,"Too small : width resized to 500\n");
								WINDOWwidth = 500;
							}
							cntrl ++;
						}
						else
						{
							fprintf(stderr,"You must enter the width of the window\n");
							usage(argv[0]);
						}
						break;
					case '?':
					default:
						fprintf(stderr,"Option '%s' does not exist\n",argv[cntrl]);
						usage(argv[0]);
						break;
				}
				break;
			default:
				fprintf(stderr,"Option '%s' must be preceded by '-'\n",argv[cntrl]);
				usage(argv[0]);
				break;
		}

	if (iterations < 1)
		iterations = 1;

	if (Debug)
	{
		fprintf(stderr,"\n%s runs with the following parameters :\n",argv[0]);
		fprintf(stderr,"number of iterations is %d\n",iterations);
		fprintf(stderr,"width of the 2D window is %d\n",WINDOWwidth);
		fprintf(stderr,"height of the 2D window is %d\n",WINDOWheight);
		fprintf(stderr,"forcevector divide factor is %d\n",factor);
		if (postscript)
			fprintf(stderr,"Postscript output is made\n");
		else
			fprintf(stderr,"No postscript output is made\n");
		if (rand_coord)
			fprintf(stderr,"Random coordinates for all nodes\n");
		else
			fprintf(stderr,"No random coordinates for the nodes (only those with coordinates == 0)\n");
		if (adjust_size)
			fprintf(stderr,"width and height of the box for a node is checked\n");
		else
			fprintf(stderr,"No size_check for the boxes\n");
		fprintf(stderr,"every 2 iterations there is a resizing of the used space in the window\n");
		fprintf(stderr,"every 10 iterations there is also a sectorcheck in the window\n");
		fprintf(stderr,"every 10 iterations there are (max. 5) overlapchecks for the boxes in the window\n\n");
	}

	time(&UTC1);
	fprintf(stderr,"%s started at : %s\n",argv[0], ctime(&UTC1));

	if (Debug)
		fprintf(stderr,"Starting to read the nodes\n\n");

	/* Scan the input for hostnames with their coordinates and sizes,
	   build-up the hoststructure (with function addnodestructure) */
	/* Scan the input for connections (links) with their two sites ,
	   build up the linkstructure (with function addlinkstructure) */

	while (gets(line))
	{
		contr = sscanf(line,"%s",dummystr);

		if ( ((length_int != 0) && (strcmp(dummystr,interfacestring) == 0))
		   || (strcmp(dummystr,"interface=") == 0)
		   || (strcmp(dummystr,"Interface=") == 0) )

		/* get the connections between nodes */

		{
			if (Debug)
				printlink = TRUE;

			contr = sscanf(line,"%s %s %s",dummystr,site_1,site_2);
			if (contr == 3)
			{
				link = addlinkstructure(link,interfacestring,site_1,site_2);
			}
			else
			{
				fprintf(stderr,"addlink: wrong data (%s %s)\n",site_1,site_2);
				continue;
			}
		}

		else /* get the node : id, name, initial coordinates and sizes */
		{
			if (Debug)
				printhost = TRUE;

			/* skip the interfaces for the node structure */

			if ( (strncmp("interface:",line,10) != 0) && (strncmp("ip.interface:",line,12) != 0) )
			{
				contr = sscanf(line,"%s %s %d %d %d %d",id,name,&x_coord,&y_coord,&width,&height);
				if (contr == 6)
					host = addnodestructure(host,id,name,x_coord,y_coord,width,height,0,FALSE);
				else
				{
					fprintf(stderr,"addhost: wrong data (%s %s %d %d %d %d)\n",
						id,name,x_coord,y_coord,width,height);
					continue;
				}
			}
		}
	}

	if (number_of_nodes == 0)
	{
		fprintf(stderr,"No nodes are read\n");
		fprintf(stderr,"Checking if we have connections\n");
		printlinkstructure(link);
		exit(2);
	}
	else
		sort_the_nodestructure();

	/* print these structures on standard output */

	if (printhost)
		printhoststructure(host);
	if (printlink)
		printlinkstructure(link);

	if (Debug)
		fprintf(stderr,"Starting with the actual layout\n\n");

	fprintf(stderr,"%s: total number of nodes %d\n",argv[0],number_of_nodes);

	if (rand_coord)
		fprintf(stderr,"\nNodes are randomly placed in the window (more linked nodes near the center) !!\n\n");

	/* get the nodes without links and save them in another structure */
	/* get the nodes with only one link and save them in another structure */

	get_unlinked_nodes(adjust_size);

	fprintf(stderr,"%s: number of nodes (with more than 1 link) to process %d\n",argv[0],number_of_nodes);

	/* get the nodes with the most links and place them in center */

	fprintf(stderr,"\nPlacing nodes in the window\n");

	if (adjust_size)
	{
		fprintf(stderr,"\nadjusting the sizes for the number of nodes\n");
		fprintf(stderr,"max. boxwidth for a node will be (windowwidth / 5) : %d\n",(WINDOWwidth / 5));
		fprintf(stderr,"max. boxheight for a node will be (windowheigth / 50) : %d\n",(WINDOWheight / 50));

		if (rand_coord)
			place_linked_node((WINDOWwidth / 5), (WINDOWheight / 50), TRUE);
		else
			place_linked_node((WINDOWwidth / 5), (WINDOWheight / 50), FALSE);
	}
	else
	{
		if (rand_coord)
			place_linked_node(WINDOWwidth, WINDOWheight, TRUE);
		else
			place_linked_node(WINDOWwidth, WINDOWheight, FALSE);
	}

	/* determine maximum possible links */

	link_node_ratio = links_nodes(number_of_nodes);

	/* get the changes for attracting (linked) nodes and repelling (unlinked) nodes */
	/* only do this with nodes with more links (otherwise it will take too long to finish) */

	for (it=1; it <= iterations ; it++)
	{
		fprintf(stderr,"Doing iteration %u\n", it);
		layout_changes(link_node_ratio,factor);

		/* scale the results so we have all nodes on the whole window */
		/* with / without sectorchecking */
		/* overlapping coordinates of nodes are fixed */

		if ( (it % 10 == 0) && (it != iterations) )
		{
			allign_nodes();
			resizing(TRUE, FALSE);
		}
		else
			resizing(FALSE, FALSE);

		if ( (it % 10 == 0) || (it == iterations) )
		{
			contr = 1;	con = 1;
			while ((contr) && (con++ <= 3))
				overlap_coordinates(&contr);
		}
	}

	/* scale the results so we get a nice centered picture. finally don't do sectorchecking */
	/* final cycle (max. 10 times) overlap_check before printing the new layout */

	contr = 1;	it = 0;
	while ( (contr) && (it++ <= 10) )
	{
		if (it % 2 == 1)
		{
			allign_nodes();
			resizing(FALSE, TRUE);
		}
		overlap_coordinates(&contr);
	}

	/* place the extra nodes (more links, but only to nodes with one links) in the emptiest sector */

	place_more_linked_nodes(adjust_size);

	fprintf(stderr,"\n%s: number of nodes (added specials) %d\n",argv[0],number_of_nodes);

	/* place the nodes with one link, close to the node, with more links, it is linked to */

	place_one_link_nodes(adjust_size);

	fprintf(stderr,"%s: number of nodes (added one_link_nodes) %d\n",argv[0],number_of_nodes);

	/* place the linkless nodes on the bottomborder of the window */

	place_unlinked_nodes();

	/* make postscript outputfile : /tmp/psfile
	   and print the final results */

	if (postscript)
		make_postscript();
	print_node();
	print_link();

	time(&UTC2);
	fprintf(stderr,"\n%s ended at : %s",argv[0], ctime(&UTC2));
	fprintf(stderr,"\nSeconds used for %s: %ld\n",argv[0], (UTC2 - UTC1));
}
