/**************************************************************************************************
 *
 *	unlinked.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	July 1991.
 *	email: a89@nikhef.nl
 *
 *************************************************************************************************/

# include "layout.h"

	/* get the nodes without links. They are not needed for the iterations of the layoutprogram.
	   These nodes are finally placed in the border of the window */
	/* also the nodes with only one link are gathered and placed close to the more_linked_nodes
	   at the end of the program */

extern void get_unlinked_nodes(sizecheck)
int sizecheck;			/* check maximum sizes for the boxes */
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	struct hostentity * newhost;
	int links;				/* number of links of a node */

	newhost = NULL;
	dummyhost1 = host;
	host_zero_links = NULL;
	host_one_link = NULL;

	while (dummyhost1 != NULL)
	{
		links = 0;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->id, dummyhost2->id))
				links ++;

			dummyhost2 = dummyhost2->next;
		}

		if (Debug)
			fprintf(stderr,"node %s has %d links\n",dummyhost1->hostname,links);

		if ( (links == 0) || (links == 1) )
		{
			/* vector determination will otherwise be wrong */

 			number_of_nodes --;
			if (links == 0)
				host_zero_links = addnodestructure(host_zero_links, dummyhost1->id, dummyhost1->hostname,
								   dummyhost1->x_coord,	dummyhost1->y_coord,
								   dummyhost1->width, dummyhost1->height, 0, TRUE);
			else
				host_one_link = addnodestructure(host_one_link, dummyhost1->id, dummyhost1->hostname,
								 dummyhost1->x_coord, dummyhost1->y_coord,
								 dummyhost1->width, dummyhost1->height, 1 ,TRUE);
		}
		else
		{
			/* make temp list with nodes with TWO or more links and copy this back later */
			/* make more_linked_nodes look more important */

			if ( (links > 2) && (links <= 5) )
				dummyhost1->width += 0.33 * DEFAULTwidth;
			else if ( (links > 5) && (links < 10) )
			{
				dummyhost1->width += 0.67 * DEFAULTwidth;
				if (dummyhost1->height <= DEFAULTheight)
					dummyhost1->height += 10;
			}
			else if (links >= 10)
			{
				dummyhost1->width += 1.00 * DEFAULTwidth;
				if (dummyhost1->height <= DEFAULTheight)
					dummyhost1->height += 10;
			}
			if (sizecheck)
			{
				if (dummyhost1->width > (WINDOWwidth / 5) )
					dummyhost1->width = WINDOWwidth / 5;
				if (dummyhost1->height > (WINDOWheight / 25) )
					dummyhost1->height = WINDOWwidth / 25;
			}

			newhost = addnodestructure(newhost,dummyhost1->id, dummyhost1->hostname, dummyhost1->x_coord,
						  dummyhost1->y_coord, dummyhost1->width, dummyhost1->height, links, TRUE);
		}

		dummyhost1 = dummyhost1->next;
	}

	/* after the complete cycle : make changes known to the original hoststructure */

	host = newhost;

	/* eventually resize the box for the more_links_node so it will not be smaller
	   than the largest box of a/the one_link_node(s) it is connected to */

	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		dummyhost2 = host_one_link;
		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->id, dummyhost2->id))
			{
				if (dummyhost1->width < dummyhost2->width)
					dummyhost1->width = dummyhost2->width;
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}
}


	/* place the unlinked nodes in the bottomborder of the window */

extern void place_unlinked_nodes()
{
	struct hostentity * dummyhost1;	/* node without links to determine the coordinates for */
	int x_offset = 25;		/* starting x_value */
	int unlinked_y_coord = 75;	/* y_coordinate for unlinked nodes */
	int unlinked_nr = 0;

	dummyhost1 = host_zero_links;

	while (dummyhost1 != NULL)
	{
		unlinked_nr ++;
		if ( (x_offset + dummyhost1->width) <= (WINDOWwidth - 50) )
		{
			dummyhost1->x_coord = x_offset;
			x_offset += dummyhost1->width + 25;
		}
		else
		{
			x_offset = 25;
			unlinked_y_coord -= (DEFAULTheight + 10);
			dummyhost1->x_coord = x_offset;
			if (unlinked_y_coord < 0)
			{
				unlinked_y_coord = 0;
				fprintf(stderr,"Too many unlinked nodes (%d) to place on WINDOWbottompart\n",unlinked_nr);
			}
		}
		dummyhost1->y_coord = unlinked_y_coord;

		printf("%s %s %d %d %d %d\n",dummyhost1->id, dummyhost1->hostname, dummyhost1->x_coord,
			dummyhost1->y_coord, dummyhost1->width, dummyhost1->height);

		if (Debug)
			fprintf(stderr,"place_unlinked_nodes: %s %s %d %d %d %d\n",dummyhost1->id, dummyhost1->hostname,
					dummyhost1->x_coord, dummyhost1->y_coord,dummyhost1->width,dummyhost1->height);
		dummyhost1 = dummyhost1->next;
	}
}
