/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/connection.c,v 1.2 1991/03/22 16:06:36 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

#include "monetd.h"
#include "general/NiceSocket.h"
#include "general/Channel.h"
#include <errno.h>
#include <sys/ioctl.h>


/**********************************************************/
/* Definitions */
/**********************************************************/

Connection ConnectionList = NULL;

struct server {
	int server_port;
	int fid;
} S_server,  * Serv;

/***************************************************************************/
/*  Handles a new connection request from a client  */
/***************************************************************************/

static void HandleServConnection (fid, data)
	int fid;
	caddr_t data;
{
	struct sockaddr addr;
	int addrlen, n = 1;

	PROC ("HandleServConnection");

	if (fid != Serv->fid) {
		sys_error ("Fid's are not equal %d %d", fid, Serv->fid);
		fid = Serv->fid;
	}

	debug_message ("accepting port [%d]", Serv->fid);
	addrlen = sizeof (addr);
	fid  = accept (Serv->fid, &addr, &addrlen);

	if (fid < 0) {
		sys_error ("Could not accept, errno %d", errno);
		return;
	}
	/* Non blocking */
	ioctl(fid, FIONBIO, &n);

	(void) create_connection (fid);
}

/**********************************************************/
/*  Create/Free new connections  */
/**********************************************************/

extern Connection create_connection (fid)
	int fid;
{
	Connection new;

	PROC ("create_connection");
	new = MonetNew (struct connection);
	if (new == NULL) {
		sys_error ("Could not allocate space for new connection");
		return NULL;
	}
	new -> requests = NULL;
	new -> fid = fid;
	new -> format = attribute_compact_ascii;
	new -> loading_entity = NULL;
	new -> next = ConnectionList;
	ConnectionList = new;
	write_timestamped_log (PingerLog, "ACCEPT", "Accepted connection on port [%d]", fid);
	new -> channel = Channel_create (fid, parse_message, free_connection, (caddr_t) new);

	debug_message ("The created new connection [%d]", new->fid);

	return new;
}

extern void free_connection (connection)
	Connection connection;
{
	Connection c;

	PROC ("free_connection");

	debug_message ("The client seems to have disconnected, closing connection [%d]", connection->fid);
	write_timestamped_log (PingerLog, "CLOSING", "Closing connection on port [%d]", connection->fid);
	while (connection->requests != NULL)
		free_request (connection->requests);

	if (connection == ConnectionList) {
		ConnectionList = connection -> next;
	} else {
		FORALL (c, ConnectionList, next) {
			if (c->next == connection) {
				c->next = connection -> next;
			}
		}
	}
	Channel_free (connection->channel);

	MonetFree (connection);
}

/**********************************************************/
/*   Find a connection, given the fid   */
/**********************************************************/

extern Connection fid2connection (fid)
	int fid;
{
	Connection c;

	FORALL(c, ConnectionList, next)
		if (c->fid == fid)
			return c;
	sys_error ("No connection with fid %d", fid);

	return NULL;
}

/**********************************************************/
/*  Initiate and finish  */
/**********************************************************/

extern Boolean init_connections (port)
	int port;
{
	int fid;

	PROC ("init_connections");

	ConnectionList = NULL;
	fid = serversock (port);
	if (fid < 0) {
		user_error ("Can't listen on port [%d]", port);
		return False;
	}
	user_inform ("Listening on port [%d]", port);

	Serv = &S_server;
	Serv -> server_port = port;
	Serv -> fid = fid;

	(void) create_input_fid  (fid, fid_read, HandleServConnection, (caddr_t) NULL);
	return True;
}

extern void finish_connections ()
{
	Connection c;
	FORALL (c, ConnectionList, next)
		free_connection (c);
	free_input_fid (Serv->fid);
	ConnectionList = NULL;
}

/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/connection.c,v 1.2 1991/03/22 16:06:36 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

