/************************************************************************************/
/************************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/monetd.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/************************************************************************************/
/************************************************************************************/

#include "monetd.h"
#include <signal.h>
#include <setjmp.h>
#include "io/io.h"

/**********************************************************/
/*  Private Datastructures */
/**********************************************************/
/*                                                        */
/*    connections 1:N requests N:1 Entitys N:1 redirects  */
/*                                                        */
/**********************************************************/
/*                                                        */
/*  connection0  ->  request0 -> request1                 */
/*       |            ^           ^                       */
/*       |            |           |                       */
/*       |            |           +-------------->  Entity0  */
/*       |            |                              |    */
/*       |            |                              v    */
/*       |            +-------------------------->  Entity1  */
/*       |                       +---------------->  |    */
/*       |                       |                   |    */
/*       v                       v                   |    */
/*  connection1  ->  request2 -> request3            |    */
/*                    ^          ^                   |    */
/*                    |          |                   v    */
/*                    |          +--------------->  Entity3  */
/*                    |                              |    */
/*                    |                              v    */
/*                    +-------------------------->  Entity4  */
/*                                                        */
/**********************************************************/
                                             

/********************************************************************************/
/*  Globals  */
/********************************************************************************/

static Connection DummyConnection;
static jmp_buf env;
Log PingerLog;

/****************************************************************************/
/*  Fiish off  */
/****************************************************************************/

static void finish ()
{
	sys_error   ("Caught INT");
	ClassEntity_save_db (Config_entity_database_file);
	finish_request ();
	ClassEntity_finish ();
	Query_finish ();
	close_server ();
	finish_connections ();
	free_log (PingerLog);
	Monet_finish ();
	exit(0);
}

/****************************************************************************/
/*  Signal handlers   */
/****************************************************************************/

static void sig_alarm ()
{
	sys_error   ("Caught SIGALRM");
	user_inform ("Caught SIGALRM");
	exit (2);
}

static void sig_virtualalarm ()
{
	sys_error   ("Caught SIGVTALRM");
	user_inform ("Caught SIGVTALRM");
	exit (6);
}

static void sig_hangup ()
{
	sys_error   ("Caught SIGHUP");
	user_inform ("Caught SIGHUP");
	exit (3);
}

static void sig_quit ()
{
	sys_error   ("Caught SIGQUIT");
	user_inform ("Caught SIGQUIT");
	exit (4);
}


static void terminate ()
{
	sys_error   ("Caught INT");
	finish ();
}

static void resume ()
{
/*	write_log ("SIGHUP", "Restarting"); */
	longjmp( env, 1);
}

/*****************************************************************************
 * detach all associtaed terminals and chdir to the root
 *  
 *****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Monetd/RCS/monetd.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $ */
/* $Author: a38 $ */
/* $Source: //gaia/group.local/computer/monet/Monetd/RCS/monetd.c,v $ */
/* $State: Exp $ */

/* $Log: monetd.c,v $
 * Revision 1.2  1991/03/22  16:09:27  a38
 * *** empty log message ***
 *
 * Revision 1.1  1991/03/22  15:29:50  a38
 * Initial revision
 *
 * Revision 1.1  89/05/17  13:07:12  uno
 * Initial revision
 * 
 * Revision 1.1  89/05/15  01:09:06  uno
 * Initial revision
 *  */

#define BSD

/* #include <stdio.h> */
/* #include <WIDE/widelib.h> */
/* #include <signal.h> */
#ifdef BSD
#include <sys/file.h>
#include <sys/ioctl.h>
#endif BSD

detach()
{
	int fd,nfile;

#ifdef SIGTTOU
	signal(SIGTTOU,SIG_IGN);
#endif SIGTTOU
#ifdef SIGTTIN
	signal(SIGTTIN,SIG_IGN);
#endif SIGTTIN
#ifdef SIGTSTP
	signal(SIGTSTP,SIG_IGN);
#endif SIGTSTP
	if (fork() != 0) {
		exit(0);
	}
#ifdef BSD
	setpgrp(0,getpid());
	if ((fd = open("/dev/tty",O_RDWR)) >= 0) {
		ioctl(fd,TIOCNOTTY,0);
		close(fd);
	}
#else /* SYSV */
	setpgrp();
	if (fork() != 0) {
		exit(0);
	}
#endif
	nfile = getdtablesize();
	for (fd = 0 ; fd < nfile ; fd++) {
		close(fd);
	}
	open("/", O_RDONLY);
	dup2(0, 1);
	dup2(0, 2);
	chdir("/");
	umask(0);
	return;
} /* detach() */

/****************************************************************************/
/*  Handle snmp traps  */
/****************************************************************************/

extern Boolean SnmpTraps;

static void snmp_before (numbits, readbits, writebits, other, timer)
	int *numbits;
	fd_set *readbits, *writebits, *other;
	struct timeval *timer;
{
	int block;
#ifdef CMU_SNMP
	if (SnmpTraps) {
		block = 1;
		snmp_select_info(&numbits, &readbits, &timer, &block);
		/* if (block == 1)
			timer = NULL;   block without timeout */
	}
#endif /* CMU_SNMP */


}

static void snmp_after (count, readbits, writebits, other)
	int count;
	fd_set readbits, writebits, other;
{
#ifdef CMU_SNMP
	if ( ! SnmpTraps)
		return;
	if (count == 0) {
		snmp_timeout();
	} else if (count > 0) {
		snmp_read(&readbits);
	}
#endif /* CMU_SNMP */

}

/****************************************************************************/
/*   Main */
/****************************************************************************/

main (argc, argv)
	int             argc;
	char           **argv;
{
	long checked_x25_file = 0, checked_redirects = 0;
	long StatusWriteOutTimer = 0;

	/* -- Init basic monet stuff */
	Monet_init ("monetd", finish, argc, argv);

	user_inform ("Do NOT run with setuid root");

	/* -- Set up log file  */
	PingerLog = create_log (Config_program_name, 0, True);
	write_comment_log (PingerLog, "status behaviour time_of_change pings_since_change id");

	/* -- Init the network query stuff */
	Query_init ();

	/* -- Init the entity database */
	ClassEntity_init ();

	/*  -- Load default class database  */
	ClassEntity_load_db (Config_class_edb);

	init_request ();
	start_server ();
	if ( ! init_connections (Config_pinger_port)) {
		user_error ("Could not start server");
		exit (1);
	}

	/*  -- Load database  */
	ClassEntity_load_db (Config_entity_database_file);

	/*  -- Create a special connection and load in initial requests from a file  */
	DummyConnection = create_connection (-1);
	load_requests (DummyConnection, Config_monetd_init_file);

	/*  Catch miscelaneous signals for no very good reason  */
	signal (SIGVTALRM, sig_virtualalarm);
	signal (SIGALRM, sig_alarm);
	signal (SIGHUP, sig_hangup);
	signal (SIGQUIT, sig_quit);

	/*  Restart if SIGHUP received, die if Kill or interupt sent  */
	setjmp (env);               /* restart point */
	signal (SIGHUP, resume);    /* when receive SIGHUP, clear everything and */
	signal (SIGINT, terminate); /* when receive SIGINT, the daemon terminates */
	signal (SIGKILL, terminate);


#ifdef ABCDEF
	if ( g_mode == DAEMON_MODE ) {
		EPRINTF( fp_msg, "%s: being a daemon process. forking...\n", Myname);
		detach(); /* Dissociate server from controlling terminal */
		/* All file descripters are closed. */
	}
#endif

	/* -- Event loop */
	while (True) {
		long timenow;
		Cardinal total_reachable = 0;

		/* Waits for requests to come in but does not return in less than */
		/* Config_monetd_delay seconds. */
		/* Write the current status of everything every now and then */
		timenow = time (0);
		if (StatusWriteOutTimer < timenow) {
			Query_log_all (PingerLog);
			timenow = time (0);
			StatusWriteOutTimer = timenow + Config_monetd_writeout_period;
		}
		/* Check that the x121/address translation file has not been changed */
		if (checked_x25_file + Config_monetd_check_X25_file_delay < timenow) {
			check_x25_file ();
			checked_x25_file = timenow;
			timenow = time (0);
		}
		/* Periodically try to reconnect to other monitors */
		if (checked_redirects + Config_monetd_reconnect_redirects_delay < timenow) {
			reconnect_redirects ();
			checked_redirects = timenow;
			timenow = time (0);
		}
		/* Waits for requests to come in but does not return in less than */
		/* Config_monetd_delay seconds. */
		wait_for_messages (timenow, Config_monetd_delay, snmp_before, snmp_after);
		total_reachable = Query_all (PingerLog);
		return_all_results ();
		IF_TESTING {
			report_all_requests (user_inform);
			user_inform ("Total reachable is %d", total_reachable);
		}
	}

	/* NOTREACHED */
}

/************************************************************************************/
/************************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/monetd.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/************************************************************************************/
/************************************************************************************/


