.TH MONETD 1N "8 July 1991" 
.UC 4
.SH NAME
.TP
.B monetd
\-
Distributed network monitoring and database agent
.SH SYNOPSIS
.B monetd
[-p <port>] [-help [1|2]] [options]
.SH OPTIONS
.TP 8
.B -help " level"
Gives a full list of options, if \fIlevel\fR is 2 then the
value of each option is printed too.
.TP 8
.B -p " port"
sets the port number listened to by the \fImonetd(8)\fP daemon.
.TP 8
.B -testing
sets test mode, which enables a few untested and dangerous features.
.TP 8
.B -debug
sets debug mode, which produces a lot of messages.

.SH DESCRIPTION
\fIMonetd\fR monitors networks using various protocols, processes the information
into a set of status records for each object monitored, keeps logs and returns
this information to clients that request it.

.SH USE
You can run more than one \fImonetd\fR daemon for your site,
but only one for clients to connect to.
Select a machine
to run it on and enter the address of this machine and the ip port
you selected in the system configuration file \fImonet/etc/monetrc\fR
.PP
Do \fBNOT\fR run with setuid root.  It won't hurt but it won't work
either.  IP pinging is done by a separate program \fIrping\fR.
.PP
To start the daemon use the command below and expect a few error
messages.
These are just the result of it not being able to open some sockets.
The current software has not really been programmed as a daemon and so
may not behave as nicely as it should.
.sp 1
.PP
.B \fBmonetd >\& /dev/null \&\fR
.SH OPERATION
Each connection to the server may have a set of outstanding requests,
each uniquely identified by the connection number and an integer.
When the conenction is broken all its requests are removed.
On start up a special connection is made which reads its requests from
a file \fImonet/etc/monetd.conf\fR,
these will be outstanding for the life time of the process.
.PP
A number of sockets are opened at start up, if they cannot be opened
a warning will be given but otherwise the program will proceed as
normal, without its full querying capability.
.PP
Although it is possible to send out many requests at once, this
capability should be limited so as to reduce the chance of
overloading part of the network.  Currently only one query is made at a
time, to further prevent overloading a configurable delay is
introduced between each set of requests.
.PP
New requests are given priority so that users get a quick response.
It is also possible to prioritise requests by giving a \fIfrequency\fR
option with each request, specifying the minimum time between
queries.  This is important because sending out too many queries
can easily overload the network or a router.
.SH REDIRECTS
Monitoring can be distributed across a number of machines by
redirecting certain queries to selected monitors of differnet hosts.
These second monitors will
do the actual monitoring of the specified objects and pass
the status information back to the the user via the first monitor.
The scheme is recursive, so requests could be passed through a
chain of monitors.
.PP
The queries to be redirected a selected by associating a simple
regular expression with the address of a monitor.  When a 
request comes in that matches the regular expression it is
automatically redirected to the second monitor.  This process
is transparent to the initiating client.
.PP
There are a number of issues that still need to be dealt with, such as how
to distribute the redirects in an organised matter and what to do when
a monitor cannot be reached.

.SH SETUP
Have a look at the file \fImonet/etc/monetd.conf\fR.  Enter the
names of all the machines you want to monitor continously.

If you wanted to use a special monitor to do all the x25 monitoring
that was running on \fInikhefh.nikhef.nl, port 4444\fR you would enter
the line,
.PP
\fIredirect nikhefh.nikhef.nl+4444 x25:*\fR
.PP
If there were a monitor running on \fImon.nerc.hc\fR that covered the whole
of the \fIherc.hc\fR site, you might want to use this for monitoring rather
than doing it directly.  Use,
.PP
\fIredirect mon.herc.hc+9999 *herc.hc*\fR
.PP
To monitor \fIhef-router.nikhef.nl\fR you first have to select
a unique request number (unique in this file), 23 for example,
and then enter,
.PP
\fI
request23 ip.host:hef-router.nikhef.nl\fR
.PP
If you wanted a special warning if it goes down for more than 5 minutes
you would convert 5 minutes into seconds, 300, and enter,
.PP
\fIrequest23 ip.host:hef-router.nikhef.nl critical300\fR
.PP
No spaces between \fIrequest\fR or \fIcritical\fR and the number
following.

.SH X.25 ADDRESS TABLE
X.25 address/name translation is done via a file \fImonet/etc/x25hosts\fR.
This is because there is no standard way of mapping names and although
each system has its own way of doing name lookups these either, do
not provide enough information or are broken.
.PP
The format is simple, one entry per line with fields separated by
spaces or tabs, for example,
.PP
\fB<name>   <dte address>  [<user data>]\fR
.PP
\fBnikhefh.nikhef.nl  204129004353\fR
.PP
A third field is also allowed, this specifies the \fIuser data\fR.
This is given by a string of characters and decimal values,
optionally surrounded by double quotes, for example
the string of 4 bytes with values 3,4,2,255 might be given by,
.PP
\fB``\\3\\4\\2\\255''\fR
.PP
Printable characters can be entered directly,
.PP
\fB``dummy user data''\fR
.PP
Or a combination of both
.PP
\fB``\\43ABC\\3Q''\fR
.PP
The default \fIuser data\fR value will be for \fIpad\fR, however some routers
are configured to have a special value for monitoring purposes.

.SH LOGS
All logs are kept in the standard log directory.  \fIMonetd\fR stores
all its logs in the same file.  Each line contains a record the
first field in each line is the type of query.  Fields are
separated by spaces or tabs.
Dates are written as two fields, \fIyy/mm/dd\fR and \fIhh/mm/ss\fR.
Comments are also inserted into the text.
.PP
Only changes in the state of an object are recorded.  This is
done using two records giving the previous and the new state.
After a configurable period the state of everything is written out,
once a day by default.
.PP
There is a problem with how to record changes.  If we ping an IP
host 20 times in an hour and it was down for all of them then we
are pretty sure that it has been down for an hour.  If we only
pinged once at the start of the hour and once at the end we are
in some doubt as to wether or not it has been up at all during the
period.  Therefore, not only should status changes be recorded but also
some metric indicating the frequency of monitoring or a reliability
statistics attached to each period.


.SH CLIENT/SERVER PROTOCOL
The protocol is not really a protocol as such because it is very dumb.
All communications is done in either a compressed or uncompressed
form of ascii.  This aids debugging and also allows the agents
to be accessed via \fItelnet\fR.
Each client has a set of outstanding requests for the duration of the
connection.  These in
no way interfere with other clients, except that the same request
given by more than one client will generate only one network query.
.TP 4
\fBhello <host> <port>\fR
identifies the client, if this matches the servers address and port
then the connection is closed because this is obviously a cycle.
The situation occurs when chaining redirects.
.TP 4
\fBdelete <#id>\fR
\-
remove a request.
.TP 4
.B clear
\-
remove all requests from this connection.
.TP 4
.B close
\-
close this connection
.TP 4
.B select<name>
\-
return full status information on the specified entity.
.TP 4
\fBredirect<host+port pattern>\fR
	\-
	this tells the agent to chain requests on to another
	agent (possibly on some other machine), provided they
	match a wildcard pattern.
.TP 4
.B clear_redirects
\-
delete existing redirects.
.TP 4
\fBrequest<#id name <options>>\fR
\-
create a new request for this connection.   Options for
	this include:
	\fIcritical<seconds>\fR,
		specify a period of seconds after which an unreachable
		object is flagged as critical, zero means never;
	and \fIfrequency<seconds>\fR, minimum time between queries.
.TP 4
.B compact
\-
return results in compact ascii.
.TP 4
.B ascii
\-
return results in expanded ascii.

.SH FILES
~/.monetdrc             user configuration file
.br
/etc/monet/monetdrc     configuration file
.br
/etc/monet/x25host      X.25 translation table
.br
/etc/monet/monetd.conf  A list of monitoring request read in at start up.
.br
/etc/monet/mib.txt      SNMP Variable definition in ASN.1
.br
/etc/rping              Like ping but only sends one query.
.fi
.SH "SEE ALSO"
monet(1N), monetrc(5N), variables(5N), libquery(3N), rping(1N)
.SH BUGS
\- There is no security in the protocol and no checking for
redirect cycles.
There is no notification of errors in requests or responses.
.SH AUTHOR
James Barr, NIKHEF-H, Amsterdam

