/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/*  Parse incoming instructions */
/**********************************************************/

#include "monetd.h"

static Boolean 	CloseConnection;

/**********************************************************/
/*  Parse/create new request */
/**********************************************************/

static Boolean parse_request (connection, text)
	Connection connection;
	String text;
{
	Request new;
	Entity entity;
	RequestId id = 0;
	char title [512], options[512]; /* !!! */
	int cc;
	
	cc = sscanf (text, "request%lu %s %[^\n]", &id, title, options);
	if (cc <= 0) {
		return False;
	} else if (cc < 2) {
		user_error ("Bad request [%s]", text);
	} else if (cc == 3) {
		make_request (connection, title, id, options);
	} else {
		make_request (connection, title, id, NULL);
	}
	return True;
}

/**********************************************************/
/*  Parse/add a redirect message, for certain requests    */
/*  always use another monitor.                           */
/**********************************************************/

static Boolean parse_redirect (connection, text)
	String text;
	Connection connection;
{
	int i, cc;
	char host[512], regexp[512];
	unsigned int port;
	RedirectRule rule;

	cc = sscanf (text, "redirect %[^ \t\n+]+%u %[^\n]", host, &port, regexp);
	for (i=MonetStrlen(regexp)-1; i>=0 && SPACEORTAB(regexp[i]) ; i--);
	regexp[i+1] = NULL;

	if (cc < 1) {
		return False;
	} else if (cc != 3) {
		user_error ("Invalid redirect instruction [%s]", text);
	} else if (rule = create_redirect (host, port, regexp)) {
		debug_message ("Will redirect [%s] to [%s+%u]", regexp, host, port);
		Redirect_say_hello (rule);
	} else {
		user_error ("Cannot redirect [%s] to [%s+%u]", regexp, host, port);
	}
	return True;
}

/**********************************************************/
/*  Parse/delete a request  */
/**********************************************************/

static Boolean parse_delete (connection, text)
	String text;
	Connection connection;
{
	int cc;
	RequestId id;
	Request request;

	cc = sscanf (text, "delete%lu", &id);
	if (cc != 1)
		return False;

	request = id2request (connection, id);
	if (request == NULL)
		user_error ("Request %d:%d does not exists", connection->fid, id);
	else
		free_request (request);

	return True;
}
/**********************************************************/
/*  Parse/select full information on a request  */
/**********************************************************/

static Boolean parse_select (connection, text)
	String text;
	Connection connection;
{
	int cc;
	RequestId id;
	Request request;

	cc = sscanf (text, "select%lu", &id);
	if (cc != 1)
		return False;

	request = id2request (connection, id);
	if (request == NULL)
		user_error ("Request %d:%d does not exists", connection->fid, id);
	else
		return_selected (request);
	return True;
}

/**********************************************************/
/*  Parse/clear all requests on a connection  */
/**********************************************************/

static Boolean parse_clear (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("clear", text, 5) != 0)
		return False;

	while (connection->requests != NULL)
		free_request (connection->requests);

	return True;
}

/**********************************************************/
/*  Parse/clear all redirects      */
/**********************************************************/

static Boolean parse_clear_redirects (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("clear_redirects", text, 16) != 0)
		return False;
	clear_redirects ();
	return True;
}

/**********************************************************/
/*  Parse format  */
/**********************************************************/

static Boolean parse_ascii (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("ascii", text, 5) != 0)
		return False;

	connection->format = attribute_ascii;
	return True;
}

static Boolean parse_compact (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("compact", text, 6) != 0)
		return False;

	connection->format = attribute_compact_ascii;
	return True;
}

static Boolean scan (message, id)
	String *message;
	Cardinal *id;
{
	String s = *message;

	while (!isdigit(*s)) {
		if (*s == NULL)
			return False;
		s++;
	}
	for (*id = 0; isdigit(*s); s++)
		*id = (*id) * 10 + (Cardinal) (*s - '0');
	*message = s;
	return True;
}

static Boolean parse_set_compact (connection, text)
	Connection connection;
	String text;
{
	Cardinal id;
	Request r;

	if (strncmp ("set", text, 6) != 0)
		return False;
#ifdef ABCD
	text += 4;
	while (scan (&text, &id)) {
		if (*text == MonitorEndCode || *text == '\n')
			continue;
		do {
			FORALL (r, connection->requests, connection_next) {
				Entity entity = request2Entity (r);
				if (PINGID(entity) == id) {
					if (! Entity_decodeCompactAscii (entity, &text))
						user_error ("Couldn't parse status [%s]", text);

				}
			}
		} while (*text == MonitorSepCode);
		for (; *text != MonitorEndCode && *text != NULL; text++);
	}
#endif
	return True;
}

/**********************************************************/
/*  Parse/say hello  */
/**********************************************************/

static Boolean parse_hello (connection, text)
	Connection connection;
	String text;
{
	char host [512];
	int port;
	int cc;
	if ((cc = sscanf (text, "hello %s %d", host, &port)) != 2)
		return False;

	if (EQS(host, Config_local_host) && port == Config_pinger_port) {
		user_error ("Attempt to redirect to local agent [%s+%d]", host, port);
		CloseConnection = True;
		return True;
	}
	debug_message ("The Client Says Hello"); 

	return True;
}

/**********************************************************/
/*  Parse/shutdown */
/**********************************************************/

static Boolean parse_shutdown (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("shutdown", text, 8) != 0)
		return False;

	/*  !!! Shutdown */
	return True;
}

/**********************************************************/
/*  Parse/close connection */
/**********************************************************/

static Boolean parse_close (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("close", text, 5) != 0)
		return False;
	debug_message ("Requesting close connection [%d]", connection->fid);
	CloseConnection = True;
	return True;
}

/**********************************************************/
/*  Parse incoming messages  */
/**********************************************************/

extern Boolean parse_message (connection, request)
	Connection connection;
	String request;
{
	static Boolean loading = False;

	CloseConnection = False;
	debug_message ("Message on connection [%d], [%s]", connection->fid, request);
	if (connection == (Connection) 0) {
		sys_error ("Connection is NULL");
		return False;
	} else if (parse_request  (connection, request)) {
	} else if (parse_select   (connection, request)) {
	} else if (parse_request  (connection, request)) {
	} else if (parse_delete   (connection, request)) {
	} else if (parse_hello    (connection, request)) {
	} else if (parse_redirect (connection, request)) {
	} else if (parse_clear_redirects (connection, request)) {
	} else if (parse_clear    (connection, request)) {
	} else if (parse_shutdown (connection, request)) {
	} else if (parse_ascii    (connection, request)) {
	} else if (parse_compact (connection, request)) {
	} else if (parse_close (connection, request)) {
	} else if (parse_set_compact (connection, request)) {
	} else if (Entity_load_line (&(connection->loading_entity), request)) {
		loading = True;
	} else {
		user_error ("Could not parse message [%s]", request);
	}
	if (CloseConnection)
		return False;
	if (! loading)
		connection->loading_entity = NULL;
	return True;
}

/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/
