/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/FileServer/RCS/request.c,v 1.1 1991/05/06 08:17:59 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

#include "general/General.h"
#include "connection.h"
#include "general/MapServer.h"

#include <strings.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/dir.h>
#include <sys/errno.h>
extern int errno;

/**********************************************************/
/**********************************************************/
/*  Parse incoming instructions */
/**********************************************************/
/**********************************************************/

static void send_file (file, chan)
	FILE * file;
	Channel chan;
{
	if (file) {
		char buffer [1024]; /* !!! */
		while (fgets(buffer, sizeof (buffer), file)) {
			Channel_send (chan, DOING_TRANSFER);
			Channel_send (chan, buffer);
			Channel_end_of_record (chan);
		}
		Channel_send (chan, END_TRANSFER);
	} else {
		Channel_send (chan, ERROR_TRANSFER);
	}
	Channel_end_of_record (chan);
}

/**********************************************************/
/*  Parse/read a request  */
/**********************************************************/

static Boolean parse_read (connection, text)
	String text;
	Connection connection;
{
	FILE * f;

	if (strncmp (text, "read ", 5) != 0)
		return False;
	if (f = MapServer_fopen_load (text+5)) {
		send_file (f, connection->channel);
		fclose (f);
	} else {
		Channel_send (connection->channel, LOCKED_TRANSFER);
		Channel_end_of_record (connection->channel);
	}
	return True;
}

/**********************************************************/
/*  Parse/edit a request  */
/*  !!! Does not handle version numbers  */
/**********************************************************/

static Boolean parse_save (connection, text)
	String text;
	Connection connection;
{
	if (strncmp (text, "save ", 5) != 0)
		return False;
	if (connection->savefile = MapServer_fopen_save (text+5)) {
	} else {
		Channel_send (connection->channel, LOCKED_TRANSFER);
		Channel_end_of_record (connection->channel);
		return False;
	}
	return True;
}

/**********************************************************/
/*  Parse/dir */
/**********************************************************/

static Boolean parse_dir (connection, text)
	String text;
	Connection connection;
{
	DIR * dirp;
	struct direct *dp;
	String name = MonetFilename (Config_map_directory);

	if (strncmp ("dir", text, 3) != 0)
		return False;

	dirp = opendir(name);
	if (dirp) {
		user_inform ("Listing map directory [%s]", name);
		for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
			Channel_send (connection->channel, DOING_TRANSFER);
			Channel_send (connection->channel, dp->d_name);
			Channel_end_of_record (connection->channel);
			user_inform ("  %s", dp->d_name);
		}
		Channel_send (connection->channel, END_TRANSFER);
		closedir(dirp);
		user_inform ("Listed map directory [%s]", name);
	} else {
		Channel_send (connection->channel, ERROR_TRANSFER);
		user_error ("Could not access directory [%s]", name);
	}
	Channel_end_of_record (connection->channel);
	return True;
}

/**********************************************************/
/*  Parse/say hello  */
/**********************************************************/

static Boolean parse_hello (connection, text)
	Connection connection;
	String text;
{
	if (strncmp ("Hello", text, 5) != 0)
		return False;

	Channel_send (connection->channel, DOING_TRANSFER);
	Channel_send (connection->channel, "Hello yourself");
	Channel_send (connection->channel, END_TRANSFER);
	Channel_end_of_record (connection->channel);
	debug_message ("The Client Says Hello"); 

	return True;
}

/**********************************************************/
/*  Parse incoming messages  */
/**********************************************************/

extern Boolean parse_message (connection, request)
	Connection connection;
	String request;
{
	user_inform ("New message accepted [%s] on connection [%d]", request, Channel2fid (connection->channel));
	if (connection == (Connection) 0) {
		sys_error ("Connection is NULL");
		return False;
	} else if (connection->savefile) {
		fputs (request, connection->savefile);
		fputc ('\n', connection->savefile);
		return True;
	} else if (parse_save (connection, request)) {
		return True;
	} else if (parse_read (connection, request)) {
	} else if (parse_hello (connection, request)) {
	} else if (parse_dir  (connection, request)) {
	} else {
		user_error ("Could not parse message [%s]", request);
		return False;
	}
	/*  Only one message at a time  */
	/*  Returning false will close the connection */
	return False;
}

/****************************************************************************/
/*  Initialise and finish  */
/****************************************************************************/

extern void init_request ()
{
}

extern void finish_request ()
{
}

/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/FileServer/RCS/request.c,v 1.1 1991/05/06 08:17:59 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/
