#include <stdio.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netdb.h>

extern int errno;
extern char *rindex();
extern char *inet_ntoa();

int wait_sock();
int check_packet();
u_short checksum();
char *pr_type();

int verbose = 0;			/* verbose output */
int attempts = 3;			/* number of retries */
unsigned timeout = 2;			/* timeout in secs */
long timelimit;				/* safety limit */
char *program;				/* program name */

#define EX_ALIVE 0			/* host is alive */
#define EX_DEAD  1			/* host is dead */
#define EX_USAGE 2			/* illegal parameter */
#define EX_ERROR 3			/* system error */

main(argc, argv)
int argc;
char *argv[];
{
	int status = 0;
	register char *opt;

	if ((program = rindex(argv[0], '/')) != NULL)
		program++;
	else
		program = argv[0];

	while (argc > 1 && argv[1][0] == '-')
	{
	    for (opt = &argv[1][1]; *opt; opt++)
	    {
	    	switch (*opt)
	    	{
	    	    case 'r':
	    		if (argc > 2)
			{
	    			attempts = atoi(argv[2]);
	    			--argc; argv++;
			}
	    		break;

	    	    case 't':
	    		if (argc > 2)
			{
	    			timeout = atoi(argv[2]);
	    			--argc; argv++;
			}
	    		break;

	    	    case 'v':
			verbose = 1;
			break;

	    	    default:
	    		error("Unknown option %c", *opt);
	    		exit(EX_USAGE);
	    	}
	    }
	    --argc; argv++;
	}

	if (argc != 2)
	{
    		fprintf(stderr, "Usage: %s host\n", program);
    		exit(EX_USAGE);
	}

	timelimit = time(0) + (attempts * timeout);

	status = ping(argv[1]);
	exit(status);
}

#define	MAXPACKET	4096			/* max packet size */
#define PACKETSIZE	32			/* packet total size */
#define HDRLEN		8			/* ICMP header length */
#define DATALEN		PACKETSIZE-HDRLEN	/* data size */

ping(host)
char *host;
{
	register int s;
	register struct hostent *hp;
	static struct protoent *proto = 0;
	struct sockaddr toaddr, fromaddr;
	struct sockaddr_in *to   = (struct sockaddr_in *) &toaddr;
	struct sockaddr_in *from = (struct sockaddr_in *) &fromaddr;
	int addrlen = sizeof(struct sockaddr);
	int status;

	static u_char packet[MAXPACKET];
	register struct icmp *icp = (struct icmp *) packet;
	register struct timeval *tp = (struct timeval *) &packet[HDRLEN];
	register u_char *dp = &packet[HDRLEN+sizeof(struct timeval)];

	int cc = PACKETSIZE;
	int nc;
	int sequence = 0;
	int ident = getpid() & 0xFFFF;
	to->sin_family = AF_INET;

/*
 * Determine remote host address.
 */
	if (host == NULL)
		return (EX_ERROR);
	if ((hp = gethostbyname(host)) != NULL)
	{
		host = hp->h_name;
		bcopy(hp->h_addr, (char *)&to->sin_addr, hp->h_length);
		to->sin_family = hp->h_addrtype;
	}
	else if ((to->sin_addr.s_addr = inet_addr(host)) == (u_long)-1)
	{
		error("Unknown host: %s", host);
		return (EX_ERROR);
	}

/*
 * Allocate socket.
 */
	if (!proto && (proto = getprotobyname("icmp")) == NULL)
	{
		error("Unknown protocol: icmp");
		return (EX_ERROR);
	}

	if ((s = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0)
	{
		perror("socket");
		return (EX_ERROR);
	}

	if (verbose)
		printf("[%s] ", host), (void) fflush(stdout);

/*
 * Prepare output packet.
 * Store current time in data field after packet header.
 */
resend:
	tp = (struct timeval *) &packet[HDRLEN];
	(void) gettimeofday(tp, (struct timezone *)0);

	dp = &packet[HDRLEN+sizeof(struct timeval)];
	for (nc = sizeof(struct timeval); nc < DATALEN; nc++)
		*dp++ = nc;

	icp = (struct icmp *) packet;
	icp->icmp_type  = ICMP_ECHO;
	icp->icmp_code  = 0;
	icp->icmp_cksum = 0;
	icp->icmp_id    = ident;
	icp->icmp_seq   = sequence;
	icp->icmp_cksum = checksum((u_short *)icp, cc);

/*
 * Send probe packet to remote destination.
 */
	if ((nc = sendto(s, packet, cc, 0, &toaddr, sizeof(toaddr))) < 0)
	{
		perror("sendto");
		(void) close(s);
		return (EX_ERROR);
	}

/*
 * Wait until input packet arrives. Retry after timeout.
 */
reread:
	if ((status = wait_sock(s, timeout)) < 0)
	{
		perror("select");
		(void) close(s);
		return (EX_ERROR);
	}
	else if (status == 0)
	{
		if (++sequence < attempts)
			goto resend;
		errno = ETIMEDOUT;
		if (verbose)
			perror("recvfrom");
		status = EX_DEAD;
		goto done;
	}

/*
 * Fetch available input packet.
 * Note. On Apollo, recvfrom() will not be interrupted by an alarm().
 */
	addrlen = sizeof(fromaddr);
	if ((nc = recvfrom(s, packet, sizeof(packet), 0, &fromaddr, &addrlen)) < 0)
	{
		if (errno == EINTR)
		{
			if (++sequence < attempts)
				goto resend;
			errno = ETIMEDOUT;
		}
		perror("recvfrom");
		(void) close(s);
		return (EX_ERROR);
	}

/*
 * Analyze input packet. Reread if this is not what we want.
 */
	if ((status = check_packet(packet, nc, cc, ident, sequence, from)) != EX_ALIVE)
	{
		if (time(0) <= timelimit)
			goto reread;
	}

done:
	if (!verbose)
		printf("%s is %s\n", host, (status == EX_ALIVE ? "alive" : "dead"));

	(void) close(s);
	return (status);
}

int
wait_sock(sock, secs)
int sock;
int secs;
{
	fd_set fds;
	struct timeval wait;
	int nreply;

	wait.tv_sec = secs;
	wait.tv_usec = 0;
	FD_ZERO(&fds);
	FD_SET(sock, &fds);

	nreply = select(FD_SETSIZE, &fds, (fd_set *)0, (fd_set *)0, &wait);
	return (nreply);
}

int
check_packet(buf, nc, cc, ident, sequence, from)
u_char *buf;
int nc;
int cc;
int ident;
int sequence;
struct sockaddr_in *from;
{
	register struct ip *ip;
	register struct icmp *icp;
	struct timeval *sendtime;
	struct timeval recvtime;
	int hdrlen, triptime;
	char *host;

	(void) gettimeofday(&recvtime, (struct timezone *)0);

	from->sin_addr.s_addr = ntohl(from->sin_addr.s_addr);
	host = inet_ntoa(from->sin_addr);

	ip = (struct ip *) buf;
	hdrlen = ip->ip_hl << 2;
	if (nc < hdrlen + ICMP_MINLEN)
	{
		if (verbose)
			printf("%d bytes from %s (packet too short)\n", nc, host);
		return (EX_DEAD);
	}
	nc -= hdrlen;
	icp = (struct icmp *)(buf + hdrlen);

	if (verbose)
		printf("%d bytes from %s ", nc, host);

	if (icp->icmp_type != ICMP_ECHOREPLY)
	{
		if (verbose)
			printf("(not echo reply: %s)\n", pr_type(icp->icmp_type, icp->icmp_code));
		return (EX_DEAD);
	}

	if (icp->icmp_id != ident)
	{
		if (verbose)
			printf("(not from us: ident %d should be %d)\n", icp->icmp_id, ident);
		return (EX_DEAD);
	}

	if (icp->icmp_seq != sequence)
	{
		if (verbose)
			printf("(out of sequence: seq %d should be %d)\n", icp->icmp_seq, sequence);
		return (EX_DEAD);
	}

	if (nc != cc)
	{
		if (verbose)
			printf("(packet size error: size %d should be %d)\n", nc, cc);
		return (EX_DEAD);
	}

	sendtime = (struct timeval *)&icp->icmp_data[0];
	if ((recvtime.tv_usec -= sendtime->tv_usec) < 0)
		recvtime.tv_sec--, recvtime.tv_usec += 1000000;
	recvtime.tv_sec -= sendtime->tv_sec;
	triptime = (recvtime.tv_sec*1000) + (recvtime.tv_usec/1000);
	if (verbose)
		printf("seq=%d time=%d ms\n", sequence, triptime);
	return (EX_ALIVE);
}

char *
pr_type(type, code)
int type;
int code;
{
	static char *ctab[] =
	{
	/*  0 */	"network is unreachable",
	/*  1 */	"host is unreachable",
	/*  2 */	"destination unreachable, protocol",
	/*  3 */	"destination unreachable, port",
	/*  4 */	"destination unreachable, needfrag",
	/*  5 */	"destination unreachable, srcfail"
	};

	static char *ttab[] =
	{
	/*  0 */	"echo reply",
	/*  1 */	"ICMP 1",
	/*  2 */	"ICMP 2",
	/*  3 */	"destination unreachable",
	/*  4 */	"source quench",
	/*  5 */	"redirect",
	/*  6 */	"ICMP 6",
	/*  7 */	"ICMP 7",
	/*  8 */	"echo request",
	/*  9 */	"ICMP 9",
	/* 10 */	"ICMP 10",
	/* 11 */	"time exceeded",
	/* 12 */	"parameter problem",
	/* 13 */	"timestamp request",
	/* 14 */	"timestamp reply",
	/* 15 */	"info request",
	/* 16 */	"info reply",
	/* 17 */	"mask request",
	/* 18 */	"mask reply"
	};

	if (type < 0 || type > 18 )
		return ("unknown packet type");

	if (type == ICMP_UNREACH)
		if (code >= 0 && code <= 5 )
			return (ctab[code]);

	return (ttab[type]);
}

u_short
checksum(addr, len)
u_short *addr;
int len;
{
	register int nleft = len;
	register u_short *w = addr;
	register int sum = 0;
	u_short answer = 0;

	/*
	 *  Our algorithm is simple, using a 32 bit accumulator (sum),
	 *  we add sequential 16 bit words to it, and at the end, fold
	 *  back all the carry bits from the top 16 bits into the lower
	 *  16 bits.
	 */
	while (nleft > 1)
	{
		sum += *w++;
		nleft -= 2;
	}

	/*
	 * mop up an odd byte, if necessary
	 */
	if (nleft == 1)
	{
		*(u_char *)(&answer) = *(u_char *)w;
		sum += answer;
	}

	/*
	 * add back carry outs from top 16 bits to low 16 bits
	 */
	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */
	return (answer);
}

/*VARARGS1*/
error(fmt, a1, a2)
char *fmt;
{
	fprintf(stderr, "%s: ", program);
	fprintf(stderr, fmt, a1, a2);
	fprintf(stderr, "\n");
}
