/* $Header: //gaia/group.local/computer/monet/h/db/RCS/attribute.h,v 1.2 1991/04/22 08:01:15 a38 Exp a38 $ */

#ifndef MONET_DB_ATTRIBUTE_H_
#define MONET_DB_ATTRIBUTE_H_

typedef struct attributeID * AttributeID;

#define NULL_ATTID (AttributeID) 0

/* Functions in an attribute can acts as triggers, currently they are not implemented */
typedef enum { op_plus, op_minus, op_uminus, op_eq, op_not, op_lt, op_and, op_or} Operator;

typedef enum {type_none, type_string, type_char, type_boolean, type_integer, type_flag, type_function} AttributeType;

#define a_all 0xff
#define a_class 1
#define a_instance 2
#define a_interface 4

typedef unsigned char AttributeClass;

/* Attributes of type_flag, do not have a value, only a presence */
struct attribute {
	AttributeID id;
	AttributeType type;
	union {
		char * string;
		Boolean boolean;
		long int integer;
		struct {
			Operator op;
			Attribute args;
		} operator;
	} v;
	Attribute next;
};

typedef enum {
	attribute_unset,
	attribute_set,
	attribute_get,		/*  Use inheritance  */
	attribute_get_shallow	/*  Do not use inheritance  */
	} AttributeOperation;

typedef enum {
	attribute_ascii,
	attribute_compact_ascii,
	attribute_binary,	/*  One day  */
	attribute_asn1		/*  For sake of completeness  */
	} AttributeFormat;


#ifdef MONET_PROTOTYPES

typedef String  (* AttributeRoutine) (Entity, AttributeOperation, AttributeFormat,String);

extern AttributeID	create_attributeID (String name, StatusMask mask, char id, Boolean standard,
				AttributeType, AttributeClass, AttributeRoutine access,
				Boolean cansave, String help);
extern AttributeID	String2AttributeID	(String);
extern char *		attributeID2string	(AttributeID);
extern void		delete_attribute	(Entity, AttributeID);
extern void		add_integerAttribute	(Entity, AttributeID, int);
extern void		add_booleanAttribute	(Entity, AttributeID, Boolean);
extern void		add_stringAttribute	(Entity, AttributeID, String);
extern void		add_flagAttribute	(Entity, AttributeID);
extern Attribute	get_attribute		(Entity, AttributeID);
extern char *		get_stringAttribute	(Entity, AttributeID);
extern Boolean		present_attribute	(Entity, AttributeID);
extern char *		get_stringAttribute	(Entity, AttributeID);
extern int		get_integerAttribute	(Entity, AttributeID);
extern Entity		free_attributeList	(Entity);
extern void		init_attributes		(void);
extern void		finish_attributes	(void);
extern void		ClassAttribute_save	(FILE * f);
extern void		ClassAttribute_report	(void);

extern void		add_stringAttribute	(Entity l, AttributeID a, char * s);

#else

typedef String  (*AttributeRoutine) ();

extern AttributeID	create_attributeID ();
extern AttributeID	String2AttributeID	();
extern char *		attributeID2string	();
extern void		delete_attribute	();
extern void		add_integerAttribute	();
extern void		add_booleanAttribute	();
extern void		add_stringAttribute	();
extern void		add_flagAttribute	();
extern Attribute	get_attribute		();
extern char *		get_stringAttribute	();
extern Boolean		present_attribute	();
extern char *		get_stringAttribute	();
extern int		get_integerAttribute	();
extern Entity		free_attributeList	();
extern void		init_attributes		();
extern void		finish_attributes	();
extern void		add_stringAttribute	();

extern void		ClassAttribute_save	();
extern void		ClassAttribute_report	();



#endif

struct attributeID {
	StatusMask mask;
	Boolean standard;
	String string;  /* Name */
	char code;
	AttributeID  next;
	AttributeType type;
	AttributeRoutine access;
	AttributeClass valid;
	Boolean cansave;
	String help;
};

/* --  This lot is not used yet !!! */
typedef String (* AttributeGetRoutine)		PR(());
typedef String (* AttributeSetRoutine)		PR(());
typedef String (* AttributeUnSetRoutine)	PR(());
typedef String (* AttributeToStringRoutine)	PR(());
typedef String (* AttributeFromStringRoutine)	PR(());
typedef String (* AttributeToCompactRoutine)	PR(());
typedef String (* AttributeFromCompactRoutine)	PR(());
typedef String (* AttributeFreeRoutine)		PR(());
typedef String (* AttributeCreateRoutine)	PR(());

struct attribute_syntax {
	AttributeGetRoutine		get, get_shallow;
	AttributeSetRoutine		set;
	AttributeUnSetRoutine		unset;
	AttributeToStringRoutine	tostring;
	AttributeFromStringRoutine	fromstring;
	AttributeToCompactRoutine	tocompact;
	AttributeFromCompactRoutine	fromcompact;
	AttributeFreeRoutine		free;
	AttributeCreateRoutine		create;
};

#define CREATE_ATTRIBUTE_SYNTAX (name, code, cansave, valid) \
	{ AttributeID new; \
	new = MonetNew (struct attributeID); \
	new -> string = MonetNewString (name); \
	new -> code = code; \
	new -> get = get;                     \
	new -> get_shallow = get_shallow;      \
	new -> set = set;                       \
	new -> unset = unset;                    \
	new -> tostring = tostring;               \
	new -> fromstring = fromstring;            \
	new -> tocompact = tocompact;               \
	new -> fromcompact = fromcompact;            \
	new -> free = free;                           \
	new -> create = create;                        \
	new -> cansave = cansave;                       \
	new -> valid = valid;                       \
	AttributeID_register (new); \
	}

#endif

