/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/*  The mapper is bsed on a very simple entity oriented        */
/*  database model.  Every displayed entity Maps/Classes/      */
/*  systems/Interfaces are entitys.                            */
/*  This lot could be tidied up quite a bit.                   */
/***************************************************************/
          
#include "db/Entity.h"
#include "query/Lib.h"
#include "at_translation.h"
#include "parser.h"

/***************************************************************/
/*  Globals  */
/***************************************************************/

#define MAXCLASSDEPTH 64
EntityList Entities; /* List of all entities  */
Entity TopClass;

/********************************************************************************/
/*  The class structure, internal, private class data */
/*  We seem to have three types of class, this is not right is it */
/*  Some attributes are inherited, some aren't need to clarify this */

/*  This structure contains special class stuff, plus attributes that can only  */
/*  be inherited */
/********************************************************************************/

struct class_data {
	Boolean anonymous_name;   /*  Create a dummy name for this object */
	Boolean underlying;	/*  Objects of this class, can underly other, higher level objects */
	Boolean bending;   /* The interface can bend */
	String title_format;
	EntityList      instances;
	Shape shape;
	Translation translations;
};

#define CLASS(obj)     (obj -> class_stuff)
#define SHAPE(c)       (CLASS(c)->shape)
#define BENDING(obj)   (CLASS(obj)->bending)
#define FORMAT(c)      (CLASS(c)->title_format)
#define INSTANCES(c)   (CLASS(c)->instances)

extern Boolean isMapClass ();

/***************************************************************/
/*  An drawing is pingable if its class a ping description */
/***************************************************************/

extern Boolean isPingable (o)
	Entity o;
{
	if (Entity_isClass (o))
		return False;
	return PING(o->class) != NULL;
}

extern Entity Entity_choose_interface (from,to)
	Entity from, to;
{
	Entity class = NULL, c = NULL, superclass;

	if (from && from->class && from->class->class)
		FOR_ENTITYLIST (c, INSTANCES (from->class->class))
			if (isInterfaceClass (c) &&
				can_interface (c, from->class) && can_interface (c, to->class))
				if (class) {
					class = NULL;
					break;
				} else {
					class = c;
				}
	return class;
}

/***************************************************************/
/*  Lookup up the translation table for an entity and return   */
/*  a class.  This is used for translating one entity into     */
/*  another.                                                   */
/***************************************************************/

extern Entity Entity_lookup_translation (from_class, via_class)
	Entity from_class, via_class;
{
	return Translation_lookup (CLASS(via_class)->translations, from_class);
}

/***************************************************************/
/*  Get next sequence of non space chars in a string  */
/***************************************************************/

static Boolean get_string (result, buf)
	String * buf, result;
{
	String place, start;
	place = *buf;
	for (;*place != NULL && (*place == ' ' || !isprint (*place)); place++);
	start = place;
	for (;*place != NULL && *place != ' ' && isprint (*place); place++, result++) {
		*result = *place;
	}
	if (place == start)
		return False;
	*buf = place;
	*result = NULL;
	return True;
}

/***************************************************************/
/*  Set the translation attribute for a class */
/***************************************************************/	

extern String set_translation (class, op, format, translation)
	Entity class;
	AttributeOperation op;
	AttributeFormat format;
	String translation;
{
	char from_s[1024], to_s[1024]; /* !!! */
	String next;
	Entity from, to;
	Translation tt;

	if (! CLASS(class)) {
		sys_error ("Can't use set_transaltion on an instance");
		return NULL;
	}
	tt = CLASS(class)->translations;

	if (translation) {
		next = translation;
		while (next) {
			if ( ! get_string (from_s, &next)) {
				break;
			} else if ( ! get_string (to_s, &next)) {
				user_error ("Need two entity names for translation, not [%s]", next);
			} else if ( ! (from = Entity_find_or_create (from_s))) {
			} else if ( ! (to   = Entity_find_or_create (to_s))) {
			} else if (! Entity_isClass (from)) {
				user_error ("[%s] is not a class", Entity_title (from));
			} else if (! Entity_isClass (to)) {
				user_error ("[%s] is not a class", Entity_title (to));
			} else if (Entity_typeOf (from) != Entity_typeOf (to)) {
				String title = MonetNewString (Entity_title (to));
				user_error ("Translation is only between classes of same type [%s] [%s]",
					Entity_title (from), title);
				MonetFree (title);
			} else {
				(void) Translation_add (tt, from, to);
			}
			for (;*next != NULL && *next != CONTINUATION_CHAR; next++);
			for (;*next != NULL && *next == CONTINUATION_CHAR; next++);
		}
		return translation;
	}
	return Translation2String (tt);
}


/******************************************************************/
/*  Get shape of object                                           */
/******************************************************************/

extern Shape shapeOf_entity (class)
	Entity class;
{
	if (class == NULL)
		return DEFAULT_SHAPE;
	if (Entity_isClass(class))
		return SHAPE(class);
	return shapeOf_entity (Entity_classOf(class));
}

extern String set_shapeOf_entity (class, op, format, shape)
	Entity class;
	AttributeOperation op;
	AttributeFormat format;
	String shape;
{
	if (shape) {
		if (Entity_isClass (class))
			SHAPE(class) = String2Shape (shape);
		else
			user_error ("Can only set shape on a class not [%s]", Entity_title (class));
		return shape;
	}
	return Shape2String (SHAPE(class));
}

extern String Entity_shapeOf (class)
	Entity class;
{
	return Shape2String (SHAPE(class));
}

/*
extern String save_shape_attribute (class)
	Entity class;
{
	if (Entity_isClass(class))
		return Shape2String (SHAPE(class));

 	return NULL;
}
*/
/******************************************************************/
/*  Stuff specific to classes  */
/******************************************************************/

static void set_class_defaults (class)
	Entity class;
{
	CLASS(class)			= MonetNew (struct class_data);
	SHAPE(class)			= DEFAULT_SHAPE;
	CLASS(class) -> instances	= create_entityList ();
	CLASS(class) -> anonymous_name	= False;
	CLASS(class) -> underlying	= False;
	CLASS(class) -> bending		= True;
	CLASS(class) -> title_format	= NULL;
	CLASS(class) -> translations	= Translation_create ();
}

/******************************************************************/
/*  Find an instance given its class and name                     */
/******************************************************************/

extern Entity string2instance (superclass, name)
	char * name;
	Entity superclass;
{
	Entity o;

	if (superclass == NULL) {
		sys_error ("Instance has no class [%s]", name);
		return NULL;
	}
	FOR_ENTITYLIST (o, INSTANCES (superclass))
		if (EQS(TITLE(o), name))
			return o;
	return NULL;
}

/******************************************************************/
/*  Return unique title of an entity */
/*  Stores name in private memory, one of two buffers, this is so */
/*  it can be called twice in a printf statement without overwriting */
/*  the buffer area.  */
/******************************************************************/

extern char * Entity_title (entity)
	Entity entity;
{
	Entity o;
	static char  buf1 [2048], buf2 [2048]; /* !!! */
	static String buffer = buf1;
	int n;
	char *c, *i;
	char *list [MAXCLASSDEPTH];

	if (buffer == buf1)
		buffer = buf2;
	else
		buffer = buf1;

	if (entity == NULL) {
		sys_error ("Entity is NULL");
		return NULL;
	}
	n = 0;
	FORALL (o, entity, class) {
		list[n] = TITLE(o);
		n++;
	}
	n--;  /*  !!! Hack to prevent 'top' being included */
	for (i = buffer,n--; n>0;n--, i++) {
		for (c=list[n];*c!=NULL;c++,i++)
			*i = *c;
		if (n!=1)
			*i = '.';
		else if (Entity_isClass (entity))
			*i = '.';
		else
			/* !!! Bad style, have bucket ready before close examination */
			i--;
	}
	if (!Entity_isClass (entity)) {
		*i = ':';
		i++;
	}

	for (c=list[0];*c!=NULL;c++,i++)
		*i = *c;
	*i = (char) 0;
	return (buffer);
}

/******************************************************************/
/*  Find an entity class given its superclass and name            */
/******************************************************************/

static Entity string2subclass (superclass, name)
	char * name;
	Entity superclass;
{
	Entity o;

	if (superclass == NULL)
		return NULL;   /* !!! */
	FOR_ENTITYLIST (o, Entity_subclasses (superclass))
		if (EQS(TITLE(o), name))
			return o;
	return NULL;
}

/***************************************************************/
/*  Search for an entity by its name  */
/*  !!! Assumes unique names, must be changed for this reason  */
/***************************************************************/

extern Entity string2entity (title)
	char * title;
{
	Boolean isclass;
	Entity superclass, result;
	char *name;

	result = parse_title (title, &name, &superclass, &isclass);
	if (name == NULL)
		return result;
	return (NULL);
}

/******************************************************************/
/* Returns entity if found */
/* If part of subtype is not found, new is NULL otherwise it has the name of the last piece */
/******************************************************************/

extern Entity parse_title (title, name, superclass, isclass)
	char * title, **name;
	Entity	* superclass;
	Boolean	* isclass;
{
	char *i, *p, c;
	Entity result;

	*name = NULL;

	if ( ! title || (* title == NULL)) {
		*superclass = NULL;
		*isclass = False;
		return NULL;
	}
	title = MonetNewString (title);
	*superclass = TopClass;

	for (p=i=title; *i!= NULL; i++) {
		if (*i=='.' || *i==':') {
			c = *i;
			*i = NULL;
			if (p==NULL || *p==NULL) {
			} else if ((*superclass = string2subclass (*superclass,p)) == NULL) {
				user_inform ("Don't know subclass [%s]", p);
				MonetFree (title);
				*isclass = True;
				return NULL;
			}
			p = i+1;
			if (c==':') {
				if ((result = string2instance (*superclass,p)) == NULL)
					*name = MonetNewString (p);
				MonetFree (title);
				*isclass = False;
				return result;
			}
		}
	}
	if ((result = string2subclass (*superclass,p)) == NULL)
		*name = MonetNewString (p);
	*isclass = True;
	MonetFree (title);
	return result;
}

/******************************************************************/
/*  Is one class a subclass of another?                           */
/******************************************************************/

extern Boolean Entity_subclass (of, sub)
	Entity of, sub;
{
	Entity parent;

	if (of == NULL)
		return True;
	FORALL (parent, sub, class)
		if (parent==of) 
			return True;
	return False;
}

/******************************************************************/
/*  Find an entity class given its full name                      */
/******************************************************************/

extern Entity string2class (s)
	String s;
{
	Boolean isclass;
	Entity superclass, result;
	char *name;

	result = parse_title (s, &name, &superclass, &isclass);
	if (isclass && name == NULL)
		return result;

	return (NULL);
}

extern Boolean free_entity_interface (interface)
	Entity interface;
{
	/*  Should disconnect  */
	return True;
}

/********************************************************************************/
/*  Match an object title against standard pattern for a class to see if it is  */
/*  valid. */
/********************************************************************************/

extern Boolean is_title_valid (class, title)
	Entity class;
	String title;
{
	if (FORMAT(class) == NULL) {
	} else if (wildmat (title, FORMAT(class))) {
	} else
		return False;
	return True;
}

/*****************************************************************/
/*****************************************************************/
/*  Queries about type of class and class attributes             */
/*****************************************************************/
/*****************************************************************/

/***************************************************************/
/*  Is this a system                                           */
/***************************************************************/

extern Boolean isSystem (o)
	Entity o;
{
	if (Entity_isClass (o))
		return False;
	return (Entity_typeOf (o) == o_system);
}

/*****************************************************************/
/*  Return the format of entity titles for a given class         */
/*****************************************************************/

extern String get_titleFormat (class)
	Entity class;
{
	if (Entity_isClass (class))
		return FORMAT(class);

	return NULL;
}

/****************************************************************/
/*  */
/****************************************************************/

extern Boolean isBendingInterface (i)
	Entity i;
{
	Entity class;

	if (Entity_isClass (i))
		class = i;
	else
		class = Entity_classOf(i);

	return BENDING (class);
}

/****************************************************************/
/*  Return classes of nodes that an interface can connect to    */
/****************************************************************/

extern Entity get_fromClass (o)
	Entity o;
{
	Entity class, c;

	if (Entity_isClass (o))
		class = o;
	else
		class = Entity_classOf (o);
	FORALL (c, class, class)
		if (FROM (c) != NULL)
			return FROM (c);
	return NULL;
}

extern Entity get_toClass (o)
	Entity o;
{
	Entity class, c;

	if (Entity_isClass (o))
		class = o;
	else
		class = Entity_classOf (o);
	FORALL (c, class, class)
		if (TO (c) != NULL)
			return TO (c);
	return NULL;
}

/****************************************************************/
/*  Return the instances of a class                             */
/****************************************************************/

extern EntityList Entity_get_instances (class)
	Entity class;
{
	return INSTANCES(class);
}

/***************************************************************/

/***************************************************************/

extern Boolean has_anonymous_name (class)
	Entity class;
{
	if (class == NULL) {
		sys_error ("has_anon.. Class is NULL");
		return False;
	}
	if (Entity_isClass (class) == False) {
		sys_error ("Anonymous name only allowed for classes");
		return False;
	}
	return CLASS(class) -> anonymous_name;
}

extern Boolean no_instances_allowed (class)
	Entity class;
{
	if (class == NULL)
		return False; /* !!! Dubious */
	if (!Entity_isClass (class))
		return False;
	return	class -> instance_type == o_none;
}

extern Boolean isUnderlying (class)
	Entity class;
{
	if (class == NULL)
		return False; /* !!! Dubious */
	if (!Entity_isClass (class))
		return False;
	return CLASS(class) -> underlying;
}

extern Boolean isInterfaceClass (class)
	Entity class;
{
	return (Entity_isClass (class) && Entity_typeOf(class) == o_interface);
}

extern Boolean isMapClass (class)
	Entity class;
{
	return (Entity_isClass (class) && Entity_typeOf(class) == o_map);
}

extern Boolean isMapEntity(entity)
	Entity entity;
{
	return (Entity_typeOf(entity) == o_map);
}

/********************************************************************************/
/*  Setting type of class  and class attributes */
/*  These are used exclusively when loading */
/********************************************************************************/

extern String set_unbendingInterface (i, op, format, value)
	Entity i;
	AttributeOperation op;
	AttributeFormat format;
	String value;
{
	if (value)
		if (isInterfaceClass (i)) {
			BENDING(i) = False;
		} else {
			user_error ("[%s] is not an interface class", Entity_title (i));
		}
	if (BENDING(i) == False)
		return "True";
	return NULL;
}

extern String set_underlying (o, op, format, value)
	Entity o;
	AttributeOperation op;
	AttributeFormat format;
	String value;
{
	if (value)
		if (Entity_isClass (o)) {
			CLASS(o) -> underlying = True;
		} else {
			user_error ("Can only set underlying attribute on classes, not [%s]", Entity_title (o));
		}
	if (CLASS(o) -> underlying)
		return "True";
	return NULL;
}

extern String set_anonymous_name (o, op, format, value)
	Entity o;
	AttributeOperation op;
	AttributeFormat format;
	String value;
{
	if (value)
		if (Entity_isClass (o)) {
			CLASS(o) -> anonymous_name = True;
		} else {
			user_error ("Cannot set anononymous name on instances [%s]", Entity_title (o));
		}
	if (CLASS(o) -> anonymous_name)
		return "True";
	return NULL;
}

extern String set_instances (class, op, format, value)
	Entity class;
	AttributeOperation op;
	AttributeFormat format;
	String value;
{
	if (value) {
		if (Entity_isClass (class)) {
			if (EQS(value, "map"))
				class -> instance_type = o_map;
			else if (EQS(value, "system"))
				class -> instance_type = o_system;
			else if (EQS(value, "interface"))
				class -> instance_type = o_interface;
			else if (EQS(value, "none")) {
				class -> instance_type = o_none;
			} else {
				parse_error ("No such instance type [%s]", value);
				return NULL;
			}
			return value;
		}
	}
	switch (Entity_typeOf(class)) {
	case o_map:
		return "map";
	case o_interface:
		return "interface";
	case o_system:
		return "system";
	default:
		return "none";
	}
}

extern String set_titleFormat (class, op, format, title_format)
	Entity class;
	AttributeOperation op;
	AttributeFormat format;
	String title_format;
{
	if (Entity_isClass (class)) {
		if (format) {
			if (FORMAT(class) != NULL)
				MonetFree (FORMAT(class));
			FORMAT(class) = MonetNewString (title_format);
		}
		return FORMAT(class);
	}
	user_error ("Can only set title format in a class definition");
	return NULL;
}

/***************************************************************/
/*  Create a new entity                                        */
/***************************************************************/

static Entity basic_create (class, title)
	char * title;
	Entity class;
{
	Entity new;

	if (title == NULL) {
		sys_error ("NULL title");
		return NULL;
	}
	new = MonetNew (struct protocol_entity);

	new->class		= class;
	new->class_stuff	= NULL;
	new->standard		= False;
	new->interfaces		= create_entityList ();
	new->supercomponents	= create_entityList ();
	new->subcomponents	= create_entityList ();
	new->attributes		= (Attribute) 0;
	new -> query		= NULL;
	new -> redirect		= NULL;
	new -> requests		= NULL;
	new -> drawing		= NULL;
	new -> map		= NULL;
	new -> from		= NULL;
	new -> to		= NULL;
	V_STATUS(new)		= create_status ((caddr_t) new);
	Entity_reset_status (new);
	TITLE(new)		= MonetNewString (title);
	insert_entityList (Entities, new);

	return (new);
}

/***************************************************************/
/*  Create an Entity Instance                                  */
/***************************************************************/

extern Entity Entity_create_instance (class, title)
	char * title;
	Entity class;
{
	Entity new = NULL;

	if (class == NULL) {
		sys_error ("NULL class when creating entity");
	} else if (!Entity_isClass (class)){
		sys_error ("Not a class");
	} else if (new = basic_create (class, title)) {
		new->instance_type = Entity_typeOf(class);
		insert_entityList (INSTANCES(class), new);
		(void) try_redirect (new);
	}
	return new;
}

/***************************************************************/
/*  Create an Entity Class                                     */
/***************************************************************/

extern Entity Entity_create_class (class, title)
	char * title;
	Entity class;
{
	Entity new;
	String p;

	if (class == NULL) {
		sys_error ("superclass is NULL");
		return NULL;
	}
	for (p = title;*p != NULL; p++)
		if (*p == '.' || !isgraph (*p)) {
			user_error ("Invalid title [%s] for class", title);
			return NULL;
		}
	if (string2subclass (class, title) != NULL) {
		user_error ("[%s] already exists", title);
		return NULL;
	}
	new = basic_create (class, title);
	new->instance_type = o_none;
	set_class_defaults (new);
	insert_entityList (Entity_subclasses (class), new);

	return new;
}

/***************************************************************/
/*  Search for entity and return it if found else create a    */
/*  new one and return that.  */
/***************************************************************/

extern Entity Entity_find_or_create (string)
	String string;
{
	Entity entity, class;
	Boolean isclass;
	String title;

	if (entity = parse_title (string, &title, &class, &isclass)) {
	} else if (class == NULL) {
		user_error ("Class of entity [%s] is unknown", string);
		return NULL;
	} else if (title == NULL) {
		return False;
	} else 	if (isclass) {
		entity = Entity_create_class (class, title);
	} else if (! is_title_valid (class, title)) {
		user_warning ("Title [%s] does not match the pattern [%s] for class [%s]",
			title, get_titleFormat (class), Entity_title (class));
	} else {
		entity = Entity_create_instance (class, title);
	}
	if (!entity)
		user_error ("Could not find or create entity [%s]", string);

	return entity;
}


/***************************************************************/
/*  Free an entity                                             */
/*  This is very difficult to get right in its current form,   */
/*  A more automatic way would be to either count references   */
/*  and don't do any freeing until they are all accounted for. */
/*  Or only allow explicit two way links between things (hmm). */
/***************************************************************/

extern void Entity_free (o)
	Entity o;
{
	if (Entity_isClass (o)) {
		if (isEmpty_entityList (INSTANCES (o)) == False) {
			user_error ("Cannot delete class with instances");
		} else 	if (isEmpty_entityList (Entity_subclasses (o)) == False) {
			user_error ("Cannot delete class with subclasses");
		} else {
			if (o->class != NULL) {
				extract_entityList (Entity_subclasses(o->class), o);
			}
			free_entityList (INSTANCES(o));
			Translation_free (CLASS(o)->translations);
			/*  Remove all references to this class from the translation tables  */
			Translation_remove (o);
			MonetFree (CLASS(o));
			return;
		}
		return;
	} else {
		switch (Entity_typeOf(o)) {
		case o_none:
 			break;
		case o_interface:
 			break;
		case o_map:
			break;
		case o_system:
			break;
		default:
			sys_error ("Unknown system type [%d]", Entity_typeOf(o));
		}
		if (o->class == NULL) {
			sys_error ("Entity's class is NULL");
		} else {
			extract_entityList (INSTANCES(Entity_classOf(o)), o);
		}
	}
	free_interfaces (o);
	free_components (o);
	stop_redirect (o);
	extract_entityList (Entities, o);
	free_entityList (Entity_interfaces(o));
	free_entityList (Entity_supercomponents(o));
	free_entityList (Entity_subcomponents(o));
	free_attributeList (o);
	free_status (o->status);

	o->class= NULL;  /*  no real reason to do this !!!  */

	MonetFree (o);
}

/***************************************************************/
/*  Init and Finish                                            */
/***************************************************************/

extern void ClassEntity_init ()
{
	Entity top;

	ClassShape_init ();
	Entities = create_entityList ();
	top = basic_create ((Entity) NULL, "top");
	top -> standard			= True;
	top -> instance_type		= o_map;  /*  This is the top  */
	set_class_defaults (top);
	TopClass = top;
	init_attributes ();
	init_class ();
	Translation_init ();
	init_redirect ();
	init_gather ();  /*  !!! Should not be here really  */
}

extern void ClassEntity_finish ()
{
	finish_gather ();
	finish_redirect ();
	Translation_finish ();
	finish_class ();
	free_entityList (Entities);
	finish_attributes();
	ClassShape_finish ();
}

/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/***************************************************************/

