/******************************************************************/
/******************************************************************/
/*  $Header$  */
/******************************************************************/
/******************************************************************/

#include "general/General.h"
#include "general/Log.h"

#ifndef sun
#include <time.h>  /* !!! Seems to have been included elsewhere */
#endif

/******************************************************************/

/******************************************************************/

struct log {
	char * name;
	int maxsize;
	int append;
	FILE * fid;
};

/******************************************************************/
/*   Initailize and finish this module  */
/******************************************************************/

extern void init_logs ()
{
}

extern void finish_logs ()
{
}


/******************************************************************/
/*  Creates a new log and associated file  */
/******************************************************************/

extern Log create_log (name, maxsize, append)
	char * name;
	int maxsize;
	int append;
{
	Log new;
	FILE * fid;
	char file[1024], * filename; /* !!! */
	(void) sprintf (file, "%s/%s", Config_log_directory, name);
	filename = (char *) MonetFilename (file);
	if (append)
		fid = fopen (filename, "a");
	else
		fid = fopen (filename, "w");
	if (fid == NULL) {
		user_error ("Sorry, cannot open log file [%s]", filename);
		return (Log) 0;
	}
	user_inform ("Opened log file [%s]", filename);
	new = MonetNew (struct log);
	new -> name = MonetNewString (name);
	new -> maxsize = maxsize;
	new -> append = append;
	new -> fid = fid;
	write_timestamped_log (new, "START", "Logging %s pid:%u", Config_program_name, getpid ());
	write_comment_log (new, "Times are in GMT (Greenwich Meantime)");
	return new;
}

/******************************************************************/
/*  Closes a log file  */
/*  First writes an END record to it */
/******************************************************************/

extern void free_log (log)
	Log log;
{
	if (log == (Log) 0)
		return;
	write_timestamped_log (log, "END", "Logging");
	fflush (log->fid);
	fclose (log->fid);
	MonetFree (log->name);
	MonetFree (log);
}

/******************************************************************/

/******************************************************************/

extern void flush_log (log)
	Log log;
{
	if (log == (Log) 0) {
		return;
	}
	fflush (log->fid);
}

/******************************************************************/
/*  Produces a timestamp, always in GMT because this is not for   */
/*  local monitoring  */
/******************************************************************/

extern char * clock2text (clock)
	long clock;
{
	static char buffer [128];
	register struct tm *tm;

	tm = gmtime (&clock);
	(void) sprintf (buffer, "%d/%d/%d %02d:%02d:%02d",
		    tm-> tm_year, tm -> tm_mon + 1, tm -> tm_mday,
		    tm -> tm_hour, tm -> tm_min, tm -> tm_sec);
	return buffer;
}

extern char * time_stamp ()
{
	long    clock;

	(void) time (&clock);
	return clock2text (clock);
}


/******************************************************************/
/*  Produces a timestamp, always in GMT because this is not for   */
/*  local monitoring.   This is beautified a bit and not so precise */
/******************************************************************/

extern char * pretty_time_stamp ()
{
	static char time_string [128];
	long currenttime;

	currenttime = time (0);
	(void) sscanf (ctime (&currenttime), "%[^\n]", time_string);
	return time_string;
}

/******************************************************************/
/*  Write an entry to the log  */
/*  Format:  "KEYWORD  ... \n"  */
/******************************************************************/

extern void write_comment_log (log, s, a,b,c,d,e,f,g,h)
	Log log;
	String s;
{
	write_log (log, "-- ", s, a,b,c,d,e,f,g,h);
}


extern void write_timestamped_log (log, type, s, a,b,c,d,e,f,g,h)
	Log log;
	String s, type;
{
	static char  buffer[1023];

	sprintf (buffer, "%20s %s ", time_stamp (), s);
	write_log (log, type, buffer, a,b,c,d,e,f,g,h);
}

extern void write_log (log, type, s, a,b,c,d,e,f,g,h)
	Log log;
	String s, type;
{
	if (log == (Log) 0) {
		return;
	}
	if (log->fid == NULL)
		return;
	fprintf (log->fid, "%-8s ", type);
	fprintf (log->fid, s, a,b,c,d,e,f,g,h);
	fprintf (log->fid, "\n");
	/* Should check file size before continuing !!! */
}

/******************************************************************/
/******************************************************************/
/*  $Header$  */
/******************************************************************/
/******************************************************************/


