/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/draw/RCS/Shape.c,v 1.2 1991/05/30 14:59:39 a38 Exp a38 $ */
/******************************************************************/
/* $Log: Shape.c,v $
 * Revision 1.2  1991/05/30  14:59:39  a38
 * Combined stuff from several sources to create a single Shape class.
 * Should be much easier to add new shapes now.
 *
 * Revision 1.1  1991/05/30  14:58:21  a38
 * Initial revision
 * */
/******************************************************************/
/******************************************************************/

#include "general/General.h"
#include "general/Draw.h"
#include "general/Shape.h"
#include "bitmap/Icons.h"

/***************************************************************************/
/*  What shape to use when drawing an object                               */
/***************************************************************************/

typedef enum {
	shape_none = 0,
	shape_disc = 1,
	shape_simple = 2,
	shape_doublering = 3,
	shape_elipse = 4,
	shape_map = 5,
	shape_joiner = 6,
	shape_vertex = 7,
	shape_button = 8,
	shape_line = 9,
	shape_string = 10,
	shape_rectangle = 11,
	shape_polyline = 12,
	shape_connector = 13,
	shape_network = 14,
	shape_roundedRectangle = 15,
	shape_toprightclippedbox = 16,
	shape_workstation = 17,
	shape_jaggy = 18,
	shape_arrow = 19
#define MAXDRAWINGSHAPE 20
	}
	DrawingShape;

/***************************************************************************/

/***************************************************************************/

#define S_CONNECTOR 1
#define S_NOT_RESIZABLE 2
#define S_EQUAL_SIDES 4
#define S_IN_RECTANGLE 8
#define S_NO_LABEL 16

typedef struct shape {
	int		flags;
	String		name;
	DrawingShape	shape;
	Shape next;
	};

static Shape ShapeTable = NULL;

/************************************************************************/
/*  Shape handling routines  */
/************************************************************************/

extern String Shape2String (shape)
	Shape shape;
{
	Shape s;

	if (shape)
		FORALL (s, ShapeTable, next)
			if (shape == s)
				return s->name;
	return "simple";
}

extern Shape String2Shape (name)
	String name;
{
	Shape s;
	FORALL (s, ShapeTable, next)
		if (EQS(name, s->name))
			return s;
	user_error ("Shape [%s] is unknown", name);
	return DEFAULT_SHAPE;
}

/******************************************************************/
/*  Can a shape be resized  */
/******************************************************************/

extern Boolean Shape_resizable (shape)
	Shape shape;
{
	if (! shape)
		return True;
	if (shape->flags & S_NOT_RESIZABLE)
		return False;
	return True;
}

/******************************************************************/
/*  Is this a connector  */
/******************************************************************/

extern Boolean Shape_isConnector (shape)
	Shape shape;
{
	if (! shape)
		return False;
	return (shape->flags & S_CONNECTOR);
}

/******************************************************************/
/*  Adjust shape  */
/******************************************************************/

extern void Shape_adjust (shape, width,height)
	Shape shape;
	Dimension *width, *height;
{
	if (! shape)
		return;
	if (shape->flags & S_EQUAL_SIDES)
		if (*width > *height)
			*height = *width;
		else
			*width = *height;
}

/******************************************************************/
/*  Prepare and draw  */
/******************************************************************/

extern void Shape_prepare (shape, force_label, conn, width,height, new_width, new_height, label)
	Shape shape;
	Boolean force_label;
	Dimension width,height, *new_width, *new_height;
	ConnectorState conn;
	String label;
{
	DrawingShape drawing_shape;

	if (shape) {
		drawing_shape = shape->shape;
	} else {
		drawing_shape = shape_simple;
	}

	width = width + 10;
	height = height + 5;

	switch (drawing_shape) {
	case shape_roundedRectangle:
	case shape_toprightclippedbox:
	case shape_string:
	case shape_line:
	case shape_rectangle:
	case shape_simple:
	case shape_elipse:
		break;
	case shape_map:
		height = height + height + 14;
		break;
	case shape_button:
		height = height + 7 + 5;
		break;
	case shape_disc:
	case shape_doublering:
		width = width + 16;
		height = height + 11;
		break;
	case shape_connector:
	case shape_arrow:
	case shape_jaggy:
		break;
	case shape_network:
		width = width + 20;
		height = height + 4;
		break;
	case shape_workstation:
		if (width < workstation_width + 4)
			width = workstation_width;
		height += workstation_height + 4;
		break;
	case shape_joiner:
	case shape_vertex:
	case shape_polyline:
		sys_error("Cannot draw, shape not implemented [%d]", shape->shape);
		break;
	default:
		sys_error("Cannot draw shape unknown [%d]", shape->shape);
		break;
	}

	/* -- Resize connectors to match state */
	if (label && force_label) {
	} else if (shape && shape->flags & S_CONNECTOR) {
		switch (conn) {
		default:
		case connector_default:
			width  = 6;
			height = 6;
			break;
		case connector_invisible:
			width  = 1;
			height = 1;
			break;
		case connector_small:
			width  /= 2;
			height /= 2;
			break;
		}
	}
	/* -- Adjust width and height */
	Shape_adjust (shape, &width, &height);
	*new_width  = width;
	*new_height = height;
}

/******************************************************************/
/*  Draw a basic shape  */
/******************************************************************/

extern void Shape_draw (shape, force_label,
		border, text, reverse,
		x,y,width,height, label_width, label_height, label, label2)
	Boolean force_label;
	Shape shape;
	Shade border, text;
	Boolean reverse;
	Position x,y;
	Dimension width,height, label_width, label_height;
	String label, label2;
{
	Position label2_x, label2_y, label_x, label_y;
	Dimension diameter = width;
	DrawingShape drawing_shape;
	int flags;

	if (shape) {
		flags = shape->flags;
		drawing_shape = shape->shape;
	} else {
		flags = S_IN_RECTANGLE;
		drawing_shape = shape_simple;
	}

	if (reverse)
		Draw_fill_rectangle(border, x + 1, y + 1, width - 2, height - 2);
		
	/* Set up defaults for position of label2 */
	label2_y = y+height + label_height;
	label2_x = x+width;
	label_x = 0;
	label_y = 0;

	switch (drawing_shape) {
	case shape_disc:
		Draw_draw_rectangle(shade_grey, x, y, diameter, diameter);
		Draw_draw_circle(border, x, y, diameter);
		break;
	case shape_connector:
		break;
	case shape_arrow:
		Draw_draw_line(border, x, y, x+width/2, y+height);
		Draw_draw_line(border, x+width, y, x+width/2, y+height);
		break;
	case shape_doublering:
		Draw_draw_rectangle(shade_grey, x, y, diameter, diameter);
		Draw_draw_circle(border, x, y, diameter);
		Draw_draw_circle(border, x + 3, y + 3, diameter - 6);
		/* Move the label in a bit to the bottom left corner */
		label2_x = x+diameter - diameter/16;
		label2_y = y+diameter - diameter/16 + label_height;
		break;
	case shape_elipse:
		Draw_draw_elipse(border, x, y, width, height);
		break;
	case shape_network:
		if (height > label_height + 2 && width > label_width + 2) {
			int nx, ny;
			nx = (width - label_width) / 2;
			ny = (height - label_height) / 2;
			if (!reverse)
				Draw_draw_box(border, x, y, width, height, nx , ny);
			/* !!! 80 is a hack, should do exact checking !!! */
			if (width > label_width + 80) {
				label2_x = x+nx+label_width+10;
				label2_y = y+ny+label_height;
			}
		} else {
			Draw_fill_rectangle(border, x, y, width+1, height+1);
		}
		break;
	case shape_string:
	case shape_rectangle:
		break;
	case shape_line:
		Draw_draw_line(border, x, y, x+width, y+height);
		break;
	case shape_polyline:
		sys_error ("Cannot draw polylines");
	default:
		sys_error("Cannot draw, shape unknown");
	case shape_simple:
		break;
	case shape_workstation:
		Draw_draw_rectangle(border, x+1, y+height-label_height-4, width-2, label_height+3);
		Draw_draw_rectangle(border, x+width/2-workstation_width/2, y+2, workstation_width, workstation_height);
		/*  put bitmap here  */
		/*  */
		break;
	case shape_toprightclippedbox:
		Draw_draw_line(border, x+width-4, y, x+width, y+4);
		break;
	case shape_roundedRectangle:
		Draw_draw_roundedRectangle(border, x, y, width, height);
		break;
	case shape_map:
		Draw_fill_rectangle(border, x+2, y+2, width-4, height-4);
		label_x = x + 2;
		label_y = y + 2;
		break;
	case shape_button:
		Draw_draw_rectangle(border, x+2, y+height/2, width-4, height/2-2);
		Draw_draw_string(text, x+4, y+2+ label_height, "Button");
		x += 2;
		y += height/4;
		label_x = x + 2;
		label_y = y + height/2;
		break;
	}

	/* Draw a box around the shape */
	if (flags & S_IN_RECTANGLE)
		Draw_draw_rectangle(border, x, y, width, height);

	/* Check to see if a string fits in the rectangle available   */
	/* If so display it.  !!! Should handle vertical strings too. */
	if (label == NULL) {
	} else if (! force_label && (flags & S_NO_LABEL)) {
	} else {
		if (label_x <= 0)
			label_x = x + (width  - label_width) / 2;
		if (label_y <= 0)
			label_y = y + (height  - label_height) / 2;
		Draw_draw_string(text, label_x, label_y + label_height, label);
	}
	if (label2)
		Draw_draw_string(text, label2_x, label2_y, label2);
}

/******************************************************************/
/******************************************************************/
/*  Create and register shapes */
/******************************************************************/
/******************************************************************/

extern ConnectorState String2ConnectorState (string)
	String string;
{
	if (EQS(string, "default"))
		return connector_default;
	if (EQS(string, "small"))
		return connector_small;
	if (EQS(string, "invisible"))
		return connector_invisible;
	return connector_default;
}

static void Shape_create (name, drawing_shape, flags)
	DrawingShape drawing_shape;
	int flags;
	String name;
{
	Shape shape;
	shape = MonetNew (struct shape);
	shape -> flags = flags;
	shape -> shape = drawing_shape;
	shape -> name  = name;
	LIST_ADD (shape, ShapeTable, next);
}


extern void ClassShape_list ()
{
	Shape s;

	start_report ("Available Shapes");
	FORALL (s, ShapeTable, next)
		writeln_report ("\t%s", s->name);
}

static Boolean Init = False;
extern void ClassShape_init ()
{
	if (Init)
		return;

	ShapeTable = NULL;
	Init = True;

/*	Pixmap icon;
	Arg tmp_arg[1];
		icon = XCreateBitmapFromData(display, XtWindow(toplevel),
				workstation_bits, workstation_width, workstation_height);
*/
	Shape_create ( "none",			shape_none,		0 );
	Shape_create ( "disc",			shape_disc,		S_EQUAL_SIDES );
	Shape_create ( "doublering",		shape_doublering,	S_EQUAL_SIDES );
	Shape_create ( "elipse",		shape_elipse, 		0 );
	Shape_create ( "map",			shape_map,		S_IN_RECTANGLE );
	Shape_create ( "joiner",		shape_joiner,		S_NO_LABEL | S_CONNECTOR );
	Shape_create ( "vertex",		shape_vertex,		S_NO_LABEL );
	Shape_create ( "button",		shape_button,		0 );
	Shape_create ( "line",			shape_line,		S_NO_LABEL );
	Shape_create ( "string",		shape_string,		S_NO_LABEL );
	Shape_create ( "rectangle",		shape_rectangle,	S_NO_LABEL | S_IN_RECTANGLE );
	Shape_create ( "polyline",		shape_polyline,		S_NO_LABEL );
	Shape_create ( "roundedrectangle",	shape_roundedRectangle,	S_IN_RECTANGLE );
	Shape_create ( "toprightclippedbox",	shape_toprightclippedbox,0 );
	Shape_create ( "workstation",		shape_workstation,	S_IN_RECTANGLE );
	Shape_create ( "network",		shape_network,		0 );
	Shape_create ( "jaggy",			shape_jaggy,
			S_IN_RECTANGLE | S_NO_LABEL | S_CONNECTOR | S_NOT_RESIZABLE );
	Shape_create ( "connector",		shape_connector,
			S_IN_RECTANGLE | S_NO_LABEL | S_CONNECTOR | S_NOT_RESIZABLE );
	Shape_create ( "arrow",			shape_arrow,
			S_IN_RECTANGLE | S_NO_LABEL | S_CONNECTOR | S_NOT_RESIZABLE );
	Shape_create ( "simple",		shape_simple,		S_IN_RECTANGLE );
	Shape_create ( NULL,			shape_none,		0 );
}

extern void ClassShape_finish ()
{
	Shape shape;
	if ( ! Init)
		return;

	Init = False;
	FORALL(shape, ShapeTable, next)
		MonetFree (shape);
	ShapeTable = NULL;
}

/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/draw/RCS/Shape.c,v 1.2 1991/05/30 14:59:39 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/
