#ifndef MONET_MAP_DRAWING_H
#define MONET_MAP_DRAWING_H

/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/Drawing.h,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************************/
/***************************************************************************/

#include "db/Entity.h"
#include "general/SimpleList.h"

/***************************************************************************/
/*  List of drawings                                                       */
/***************************************************************************/

typedef SimpleList DrawingList;
#define extract_drawingList(list,o)	extract_simpleList (list, (caddr_t) o)
#define Drawing_createList()		(DrawingList) create_simpleList ()
#define reset_drawingList(list)		reset_simpleList (list)
#define Drawing_freeList(list)		free_simpleList (list)
#define insert_drawingList(list, o)	insert_simpleList(list, (caddr_t) o)
#define isEmpty_drawingList(list)	isEmpty_simpleList(list)
#define getN_drawingList(list, n)	(Drawing) getN_simpleList(list,n)
#define first_drawingList(list)		(Drawing) first_simpleList(list)
#define next_drawingList(list)		(Drawing) next_simpleList(list)
#define sizeOf_drawingList(list) 	sizeOf_simpleList(list)
#define second_drawingList(list)		(Drawing) second_simpleList(list)
#define isMember_drawingList(list, o)	isMember_simpleList(list, (caddr_t) o)

#define FOR_DRAWINGLIST(o, list) for (o=first_drawingList (list); o != NULL; o=next_drawingList (list))


/***************************************************************************/
/*  Others */
/***************************************************************************/

#include "general/Draw.h"
#include "general/Shape.h"
#include "map/Label.h"
#include "map/Draw.h"
#include "map/Status.h"
#include "map/Mark.h"
#include "map/Map.h"

/***************************************************************************/
/*  A rectangle with a label                                               */
/***************************************************************************/

typedef struct monetBox {
	Position  x,y;
	Dimension width, height;
	String label;
} MonetBox;

#define COPYMONETBOX(to, from) { to.x = from.x; to.y =from.y; \
			to.width = from.width; to.height = from.height; \
			to.label = from.label; }
#define NULL_MonetBox {0,0,0,0,NULL}

/***************************************************************************/
/*** Cannot be selected as a supercomponent and normally at the same time */
/***************************************************************************/

typedef enum {
	select_none,
	select_first,
	select_super,
	select_normal
	}
	SelectState;

/***************************************************************************/
/*  Various attributes for the drawings                                    */
/***************************************************************************/

typedef enum {
	show_emphasised,
	show_deemphasized,
	show_normal,
	show_hiden
	}
	ShownState;

typedef enum {
	extra_none,
	extra_left,
	extra_leftbelow,
	extra_border,
	extra_double_border
	}
	ExtraDrawing;
#define extra_default extra_none

typedef enum {
	flag_none,
	flag_down,
	flag_halfmast,
	flag_up
	 }
	ExtraFlag;

typedef enum {
	extra2_none,
	extra2_crossonce,
	extra2_crosstwice,
	extra2_highlight,
	extra2_upFlag,
	extra2_downFlag,
	extra2_lowlight,
	 }
	Extra2Drawing;


typedef enum {
	topright_none,
	topright_cross,
	topright_upFlag,
	topright_downFlag,
	topright_T,
	topright_P,
	topright_C,
	topright_U,
	 }
	ExtraTopRight;

/***************************************************************************/
/*  The drawing structure                                                  */
/*  Each Drawing.has exactly one entity                                    */
/***************************************************************************/

struct drawing {

	/***************************************************************************/
	/*  Each drawing is an element of a map, a map can be a drawing to         */
	/***************************************************************************/

	Map map;		/*  Pointer to the Map object that this is displayed on */
#define Drawing_mapOf(d)	d->map

	/***************************************************************************/
	/*  If this has children  */
	/***************************************************************************/

	Map submap;

	/***************************************************************************/
	/*  A couple of general purpose selection lists are provided               */
	/***************************************************************************/

	Boolean		found;		/* On found_list */
#define FOUND(o)	(o->found)
	SelectState	select;
#define SELECT(o)	(o->select)

	/***************************************************************************/
	/* Has the system geometry been changed from its default */
	/***************************************************************************/

	Boolean		resized;

	/***************************************************************************/
	/*  Position on the map, !!! Should be a monet box                         */
	/***************************************************************************/

	Dimension	base_height, base_width;

	Position	y,x;
	Dimension	stretch_width, stretch_height;
	String		label;

	Position	layout_x, layout_y;	/* A vector used by the layout routines */

	MonetBox	list;  /* Position when listed */
	MonetBox	backup;  /* Previous position */

	/***************************************************************************/
	/*  State attributes                                                       */
	/***************************************************************************/

	String		label2;		/* Bottom right corner */
	LabelState	labelState;
	ShownState	shown;

	/***************************************************************************/
	/*  Define exactly what the drawing looks like                             */
	/*  These values are derived mainly from the state of the entity           */
	/***************************************************************************/

	Shape		shape;
	ExtraDrawing	extra;
	Extra2Drawing	extra2;
	ExtraFlag	flag;
	ExtraTopRight	topright;	
	Boolean		reverse_video;
	Shade		shade;

	/***********************************************************************/
	/*  For interfaces only  */
	/***********************************************************************/

	Drawing from, to;

	/***************************************************************************/
	/*  List of drawings attached to an entity                                 */
	/*  Each drawing must have exactly one entity                              */
	/***************************************************************************/

	Drawing		next;
	Entity		entity;
#define ENTITY(d) (d->entity)
#define Drawing_classOf(d)  Entity_classOf(d->entity)

/***************************************************************************/
};	/* End struct drawing */
/***************************************************************************/

/***************************************************************************/
/*  A list of standard entity classes                                      */
/***************************************************************************/

typedef struct  {
	Entity	interface,
		button,
		ip_host,
		ip_network,
		ip_interface,
		x25_host,
		annotation,
		annotation_line,
		annotation_text,
		annotation_rectangle;
} StandardTypes;

extern StandardTypes Std_Type;

/***************************************************************************/
/*  MACROS                                                                 */
/***************************************************************************/

#define DRAWING(o) (o)
#define LISTBOX(o) (o->list)
/* These should not be directly assigned to, routines are provided for that */
#define X(o)	(o->x)
#define Y(o)	(o->y)
#define SHOW(o)		(o->shown)
#define WIDTH(o)	(o->stretch_width)
#define HEIGHT(o)	(o->stretch_height)
#define BASE_WIDTH(o)	(o->base_width)
#define BASE_HEIGHT(o)	(o->base_height)
#define EXTRA1(o)	(o->extra)
#define EXTRA2(o)	(o->extra2)
#define EXTRAFLAG(o)	(o->flag)
#define TOPRIGHT(o)	(o->topright)
#define SHADE(o)	(o->shade)
#define REVERSEVIDEO(o)	(o->reverse_video)
#define LABEL_STATE(o)	(o->labelState)
#define LABEL(o)	(o->label)
#define LABEL2(o)	(o->label2)

#define Drawing_title(d)	Entity_title (ENTITY(d))
#define Drawing_isInterface(d)  isInterface(ENTITY(d))
#define Drawing_isClass(d)      Entity_isClass(ENTITY(d))

/***************************************************************************/
/*  Function Prototypes                                                    */
/***************************************************************************/

#ifdef MONET_PROTOTYPES

extern	void	ClassDrawing_init		(void);
extern	void	ClassDrawing_finish		(void);
extern	Drawing	Drawing_create			(String, Entity, Map, Position, Position, Dimension, Dimension);
extern	Drawing	Drawing_create_by_name		(String, Map, Position, Position, Dimension, Dimension);
extern	Boolean	Drawing_isAnnotation		(Drawing);
extern	Boolean	Drawing_isButton		(Drawing);
extern	Boolean	Drawing_isMap			(Drawing);
extern	void	Drawing_free			(Drawing);
extern	void	Drawing_unset_resized		(Drawing);
extern	void	Drawing_set_resized		(Drawing);
extern	Boolean	Drawing_prepare			(Drawing);
extern	Boolean	Drawing_position		(Drawing, Position, Position);
extern	Boolean	Drawing_dimension		(Drawing, Position, Position, Dimension, Dimension);
extern	void	Drawing_reset_list_label	(Drawing);
extern	Boolean	Drawing_isMoveable		(Drawing);
extern	void	Drawing_select			(Drawing, SelectState);
extern	void	Drawing_deSelect		(Drawing);
extern	Boolean	Drawing_isSelected		(Drawing);
extern	Drawing	Drawing_from_interface		(Drawing);
extern	Drawing	Drawing_to_interface		(Drawing);
extern	Boolean Drawing_in_box    		(Drawing, Position, Position, Dimension, Dimension);
extern	Boolean	Drawing_is_marked 		(Drawing, Position, Position);
extern	Boolean Drawing_are_linked		(Drawing, Drawing);

extern	void	Entity_select_drawings		(Entity);
extern	Drawing	get_drawing			(Entity, Map);
extern	Drawing	Drawing_find			(String, Entity, Map);
extern	void	Drawing_do_action		(Drawing);

#else

extern	void	ClassDrawing_init		();
extern	void	ClassDrawing_finish		();
extern	Drawing	Drawing_create			();
extern	Drawing	Drawing_create_by_name		();
extern	Boolean	Drawing_isAnnotation		();
extern	Boolean	Drawing_isButton		();
extern	Boolean	Drawing_isMap			();
extern	void	Drawing_free			();
extern	void	Drawing_unset_resized		();
extern	void	Drawing_set_resized		();
extern	Boolean	Drawing_prepare			();
extern	Boolean	Drawing_position		();
extern	Boolean	Drawing_dimension		();
extern	void	Drawing_reset_list_label	();
extern	Boolean	Drawing_isMoveable		();
extern	void	Drawing_select			();
extern	void	Drawing_deSelect		();
extern	Boolean	Drawing_isSelected		();
extern	Drawing	Drawing_from_interface		();
extern	Drawing	Drawing_to_interface		();
extern	Boolean Drawing_in_box    		();
extern	Boolean	Drawing_is_marked 		();
extern	Boolean Drawing_are_linked		();

extern	void	Entity_select_drawings		();
extern	Drawing	get_drawing			();
extern	Drawing	Drawing_find			();
extern	void	Drawing_do_action		();

#endif /* MONET_PROTOTYPES */

#endif

/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/Drawing.h,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************************/
/***************************************************************************/

