

#ifndef MONET_MAP_H
#define MONET_MAP_H

#include "general/Draw.h"

/***********************************************************************/
/*  Layout of elements on map                                          */
/***********************************************************************/

typedef enum {
	drawnas_map = 0,
	drawnas_list = 1 }
	DrawnAs;

/***********************************************************************/
/*  Map datastructure */
/*  Basically this is just a widget for displaying a map on.  When a   */
/*  map is changed this is reflected on the map areas.                 */
/*  This could just as well be implemented as a new widget class       */
/*  !!! This should all be private                                     */
/***********************************************************************/

struct map {
	/***********************************************************************/
	/*  If this is the top in a heirarchy then this is null  */
	/***********************************************************************/

	Drawing parent;

	/***********************************************************************/
	/*  For Positioning new elements, !!!                                  */
	/***********************************************************************/

	Position unplaced_x, unplaced_y;

	/***********************************************************************/
	/*  Have the subcomponents been saved to disc yet                      */
	/***********************************************************************/

	Boolean saved;
#define Map_is_saved(map)  (map->saved || map->entity == NULL)

	/***********************************************************************/
	/*  -- A map is basically a list of drawings  */
	/***********************************************************************/

	DrawingList elements;
#define	Map_elements(map)  ((map)->elements)
	Boolean class_map;  /* Is this map only for drawing classes ? */
#define Map_classesOnly(map) ((map)->class_map)
#define Map_set_classesOnly(map) (map)->class_map = True;

	/***********************************************************************/
	/*  The name of the map file  */
	/***********************************************************************/

	String filename;
	Entity entity;
#define Map_is_loaded(map)	((map)->entity)

	/***********************************************************************/
	/*  Widgets associated with this map area                              */
	/***********************************************************************/

	/*  Widget popup;  Not wanted !!! */
	/* Widget mapMenu; !!! */
#define map_menu(s) MAPAREA(s)->mapMenu
#define map_popup(map)    (MAPAREA(map)->popup)

	Widget mapWidget;
	Region clip_region;		/* For graphics exposure */
#define Map_clip_region(map) (map->clip_region)

	/***********************************************************************/
	/*  How the map is actually drawn                                      */
	/***********************************************************************/

	Boolean show_load;   /*  Indicate system load on the maps  */
#define Map_set_show_load(map,value) map->show_load = value;
#define Map_is_load_shown(map) (map->show_load)

	ConnectorState connectors;
	Cardinal grid_x, grid_y;
	Scale  scale;
	DrawnAs drawnas;		/* Draw a map, or just a list of systems  */
#define Map_drawnas(o)  o->drawnas
/*	Boolean expanded;	*/	/* Is the map currently Realized */

	String default_domain;	/* -- hosts in this domain will have short labels */
	String background_file;  /* -- Graphics for background */
	String background_colour;
	Cardinal	default_domain_len;


	Boolean marked;	/*  Has this map been marked  */

	/***********************************************************************/
	/* The position and dimensions of the maps X Window on the screen      */
	/***********************************************************************/
/*
	Position x,y;
	Dimension width, height;
*/
	float x_scale, y_scale;

	/***********************************************************************/
	/*  Actual dimensions of the map                                       */
	/***********************************************************************/

	Dimension xl,yl,xr,yr;
#define CLEAR_DIMENSIONS(map) map->xl = map->yl = map->xr = map->yr = 0;
#define ARESET_DIMENSIONS(map)   (map->xr && map->yr)

	/***********************************************************************/
	/*  The maps a linked together in a list  */
	/***********************************************************************/

	Map next;
};


#define FORALLMAPS(m) for (m = Map_first (); m != NULL ; m = Map_next (m))

/***********************************************************************/

/***********************************************************************/

#ifdef MONET_PROTOTYPES

typedef void (* MapChangedCallback) (Map);
extern void	ClassMap_init			(MapChangedCallback);
extern void	ClassMap_finish			(void);
extern void	ClassMap_redraw_all		(void);
extern void	ClassMap_changed		(void);

extern Map	Map_create			(void);
extern void	Map_free			(Map map);
extern void	Map_discard			(Map map);

extern Map	Map_first			(void);
extern Map	Map_next			(Map current);

extern Widget	Map_widget			(Map);
extern Map	Map_root			(Map);
extern Map	Map_get_map_for_entity		(Entity);
extern void	Map_set_domain			(Map, String);
extern String	Map_get_domain			(Map);
extern void	Map_get_dimensions		(Map, Position *, Position *, Dimension *, Dimension *);
extern void	Map_set_drawn_as		(Map, DrawnAs);
extern void	Map_set_background_file		(Map, String);
extern String	Map_get_background_file		(Map);
extern void	Map_set_background_colour	(Map, String);
extern String	Map_get_background_colour	(Map);
extern void	Map_request_redraw		(Map);
extern void	Map_position			(Map, Position, Position, Dimension, Dimension);
extern Boolean	Map_corners			(Map, Position*, Position*, Position*, Position*, Boolean force_check);
extern void	Map_adjust_corners		(Map, Drawing, Boolean adding);
extern void	Map_set_grid			(Map, Cardinal x, Cardinal y);
extern Boolean	Map_insert_drawing		(Map, Drawing, Position, Position, Dimension, Dimension);
extern Drawing	Map_insert_drawing_class	(Map, Entity, Position, Position);
extern void	Map_extract_drawing		(Map, Drawing);
extern void	Map_extract_all_drawings	(Map);
extern Boolean	Map_can_insert_drawing		(Map, Entity);
extern Boolean	Map_save			(Map);
extern Boolean	Map_load			(Map, String filename);
extern String	Map_title			(Map);
extern	void	Map_backup			(Map);
extern	void	Map_restore			(Map);

extern void	Map_ListLocalDirectory		(String name);
extern Boolean	Map_ListRemoteDirectory		(String host, int port);

extern String	drawnas2string			(Drawnas);
extern DrawnAs	string2drawnas			(String);

extern void	Map_set_changed			(Map);
extern void	Map_disable_set_changed		(void);
extern void	Map_enable_set_changed		(void);

extern Drawing Map_get_marked (Map *map,
	Position *x_map, Position *y_map, Position *x_win, Position *y_win,
	Dimension *w_win, Dimension *h_win);

extern Drawing Map_actual2map (	Map *map, 
	Position *x_map, Position *y_map,
	Dimension *w_map, Dimension *h_map);

extern Boolean Map_select_in_box (Map map,
	Position xloc, Position yloc,
	Dimension width, Dimension height);

extern void Map_adjust_for_grid (Map,
		Position x, Position y,
		Position *newx, Position *newy);

#else

typedef void (* MapChangedCallback) ();
extern void	ClassMap_init			();
extern void	ClassMap_finish			();
extern void	ClassMap_redraw_all		();
extern void	ClassMap_changed		();

extern Map	Map_first			();
extern Map	Map_next			();

extern Map	Map_create			();
extern void	Map_free			();
extern void	Map_discard			();

extern Widget	Map_widget			();
extern Map	Map_root			();
extern Map	Map_get_map_for_entity		();
extern void	Map_set_domain			();
extern String	Map_get_domain			();
extern void	Map_get_dimensions		();
extern void	Map_set_drawn_as		();
extern void	Map_set_background_file		();
extern String	Map_get_background_file		();
extern void	Map_set_background_colour	();
extern String	Map_get_background_colour	();
extern void	Map_request_redraw		();
extern void	Map_position			();
extern Boolean	Map_corners			();
extern void	Map_adjust_corners		();
extern void	Map_set_grid			();
extern Boolean	Map_insert_drawing		();
extern Drawing	Map_insert_drawing_class	();
extern void	Map_extract_drawing		();
extern void	Map_extract_all_drawings	();
extern Boolean	Map_can_insert_drawing		();
extern Boolean	Map_save			();
extern Boolean	Map_load			();
extern String	Map_title			();
extern	void	Map_backup			();
extern	void	Map_restore			();

extern void	Map_ListLocalDirectory		();
extern Boolean	Map_ListRemoteDirectory		();

extern String	drawnas2string			();
extern DrawnAs	string2drawnas			();

extern void	Map_set_changed			();
extern void	Map_disable_set_changed		();
extern void	Map_enable_set_changed		();

extern Drawing	Map_get_marked			();
extern Drawing	Map_actual2map			();
extern Boolean	Map_select_in_box		();
extern void	Map_adjust_for_grid		();

#endif

extern Map CurrentMap, MarkedMap;

                         

#endif

