#ifndef XMONET_MARK_H
#define XMONET_MARK_H

/***************************************************************/
/*  This module maintains a set of lists of marked drawings     */
/*  It is very convenient to keep this lot together            */
/*  Some of the lists have special rules, such as only one     */
/*  Member at a time.                                          */
/*  This module is responsible for updating the display after  */
/*  changes.                                                   */
/***************************************************************/


/***  The supported lists of marked drawings ***/
typedef enum {m_user, m_found, m_supercomponent, m_class, m_hidden} MarkedList;


/***  General routines for this module ***/
extern void	init_mark ();
extern void	finish_mark ();
extern void	totally_unmark ();	/* Drawing */

/***************************************************************/
/***  The core marking routines ***/
/***************************************************************/

#ifdef MONET_PROTOTYPES
extern Drawing	top_marked		(MarkedList);
		/*  Return the most recently marked drawing in the list */
extern Cardinal	numberOf_marked (MarkedList);
extern void	free_marked		(MarkedList);
extern Drawing	next_marked		(MarkedList);
extern Drawing	getN_marked		(MarkedList, int);
extern Drawing	second_marked		(MarkedList);
extern void	add_marked		(MarkedList, Drawing);
extern void	remove_marked		(MarkedList, Drawing);
extern void	mark_drawingList	(MarkedList, DrawingList);
		/*  Mark everything in an drawing list */
extern void	toggle_marked (MarkedList, Drawing);

#else

extern Drawing	top_marked ();
extern Cardinal	numberOf_marked ();
extern void	free_marked ();
extern Drawing	next_marked ();
extern Drawing	getN_marked ();
extern Drawing	second_marked ();
extern void	add_marked ();
extern void	remove_marked ();
/*  Mark everything in an drawing list */
extern void	mark_drawingList ();
extern void	toggle_marked ();

#endif

/***************************************************************/
/***  For marking systems selected by the user ***/
/*    The poor naming is historical !!! */
/***************************************************************/

#define current_Selection()	top_marked (m_user)
#define numberOf_Selection()	numberOf_marked (m_user)
#define free_selection()	free_marked (m_user)
#define next_selection()	next_marked (m_user)
#define getN_selection(i)	getN_marked (m_user,i)
#define second_selection()	second_marked (m_user)
#define add_selection(o)	add_marked (m_user,o)
#define remove_selection(o)	remove_marked (m_user,o)
#define selection_drawingList(list)	mark_drawingList (m_user,list)
#define toggle_selection(o)	toggle_marked (m_user,o)
#define FOR_SELECTIONS(o)	for (o=current_Selection (); o != NULL; o=next_selection ())
#define FOR_SELECTED_ENTITY(o)	{ \
		Drawing for_d; \
		FOR_SELECTIONS(for_d) { \
			o = ENTITY(for_d);
#define END_SELECTED_ENTITY	}}

/***************************************************************/
/***  Systems found by the system after some kind of query ***/
/***************************************************************/

#define first_found()		top_marked (m_found)
#define numberOf_found()	numberOf_marked (m_found)
#define free_found()		free_marked (m_found)
#define next_found()		next_marked (m_found)
#define getN_found(i)		getN_marked (m_found,i)
#define second_found()		second_marked (m_found)
#define mark_found(o)		add_marked (m_found,o)
#define unmark_found(o)		remove_marked (m_found,o)
#define found_drawingList(list)	mark_drawingList (m_found,list)
#define toggle_found(o)		toggle_marked (m_found,o)
#define FOR_FOUND(o)		for (o=first_found (); o != NULL; o=next_found ())

/***************************************************************/
/***  For selecting the supercomponent is some operations ***/
/***  Currently only one supercomponent can be selected   ***/
/***************************************************************/

#define current_supercomponent()	top_marked (m_supercomponent)
#define free_supercomponent()		free_marked (m_supercomponent)
#define unmark_supercomponent(o)	remove_marked (m_supercomponent,o)
/* mark_supercomponent will currently replace the previously marked */
/* supercomponent */
#define mark_supercomponent(o)	add_marked (m_supercomponent,o)
#define toggle_supercomponent(o)	toggle_marked (m_supercomponent,o)

/***  For selecting a class in the class map              ***/
/***  Currently only one class can be selected at a time  ***/

#define current_class()		top_marked (m_class)
#define free_marked_class()	free_marked (m_class)
#define unmark_class(o)		remove_marked (m_class,o)
/* mark_class will currently replace the previously marked class */
#define mark_class(o)		add_marked (m_class,o)
#define toggle_class(o)		toggle_marked (m_class,o)

#endif

