/***************************************************************/
/***************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/status.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************/
/***************************************************************/

#include "Drawing.h"

/***************************************************************/
/*   Do this when the status of an drawing changes              */
/***************************************************************/

extern void status_changed_trigger (before, status, data)
	VStatus before, status;
	caddr_t data;
{
	StatusMask mask = status->mask;
	Entity o = (Entity) data;
	Boolean beeper = False;
	Drawing d;

	PROC ("status_changed_trigger");
	if (mask == StatusNoneMask)
		return;
	if (o == NULL) {
		sys_error ("Object is NULL");
		return;
	}
	FORALL(d, o->drawing, next) {
		Drawing_unmoved_undraw (d);
	}
	if (mask & StatusReachMask) {
		if (FUTURE(o) == future_planned) {
			sys_error ("Cannot set reachability on a planned system [%s]", Entity_title (o));
		} else if (STATUS(o) == status_critical) {
			beeper = True;
			user_warning ("WATCH OUT [%s] has gone CRITICAL at [%s]",
					Entity_title (o),
					pretty_time_stamp  ());
			if (CONTACT(o) != NULL)
				user_warning ("CONTACT [%s]", CONTACT(o));
		} else if (before->status == status_critical) {
			user_inform ("[%s] is no longer CRITICAL at [%s]",
					Entity_title(o),
					pretty_time_stamp  ());
		}
	}
	if (mask & (StatusCapacityMask | StatusFutureMask))
		FORALL(d, o->drawing, next)
			Map_set_changed (Drawing_mapOf(d));

	if (mask & StatusNeighbourMask) {
		/* Done in Drawing_prepare */
	}

	IF_TESTING {
		/*  Rename drawings  */
	if ((mask & StatusTitleMask) && status->title != NULL)
		/* -- reset labels of all drawings */
		FORALL(d, o->drawing, next)
			if (LABEL(d)) {
				MonetFree (LABEL(d));
				LABEL(d) = NULL;
			}
	}
	if (beeper)
		give_a_beep ();

	/************************************************************/
	/* Map the system status to the drawing attributes */
	/************************************************************/

	FORALL(d, o->drawing, next) {
		Drawing_prepare (d);
		Drawing_unmoved_redraw (d);
	}
}

/***************************************************************/
/*  Sets the official name of a system */
/***************************************************************/

extern Boolean rename_entity (entity, name)
	String name;
	Entity entity;
{
	Drawing d;
	if (string2instance (Entity_classOf(entity), name)) {
		user_inform ("Can't rename [%s] to [%s] as it already exists",
			 Entity_title(entity), name);
		return False;
	} else if ( ! is_title_valid (Entity_classOf(entity), name)) {
		user_error (
			"[%s] is an invalid label for instances of class [%s]",
			name, Entity_title (entity));
		return False;
	} else if (name && * name) {
		MonetFree (TITLE(entity));
		TITLE(entity) = MonetNewString (name);
		FORALL(d, entity->drawing, next)
			reset_label (d);
	} else {
		user_error ("Name is null");
	}
	return True;
}

extern Boolean set_alias (o, name)
	String name;
	Entity o;
{
	Drawing d;
	if (ALIAS(o) != NULL)
		MonetFree(ALIAS(o));

	ALIAS(o) = MonetNewString (name);
	FORALL(d, o->drawing, next)
		reset_label (d);
	return True;
}

/***************************************************************/
/* */
/***************************************************************/

extern void init_vstatus ()
{
	init_status (status_changed_trigger);
}

extern void finish_vstatus ()
{
	finish_status ();
}

/***************************************************************/
/***************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/status.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************/
/***************************************************************/



