
/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/translate.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/

#include "Drawing.h"
#include "Translate.h"

/******************************************************************/
/*  Translates a map of one protocol into a map of another        */
/*  Protocol.  For example given a physical map of an area, a     */
/*  corresponding IP or X.25 map would be produced                */
/******************************************************************/



/******************************************************************/
/* Init and finish */
/******************************************************************/

extern void init_translate_map ()
{
}

extern void finish_translate_map ()
{
}

/******************************************************************/
/*  Actually do it  */
/******************************************************************/

static Boolean do_translate (source, new_map, source_class, new_class)
	Map source, new_map;
	Entity source_class, new_class;
{
	Cardinal done = 0, failed = 0;
	Drawing draw;
	Entity class;

	FOR_DRAWINGLIST(draw, Map_elements (source)) {
		Entity entity = ENTITY(draw);
		if (class = Entity_lookup_translation (Entity_classOf(entity), new_class)) {
			if (Drawing_create (TITLE(entity), class, new_map, X(draw), Y(draw), WIDTH(draw), HEIGHT(draw)))
	  			done ++;
			else
				failed ++;
		} else {
			failed ++;
		}
	}
	if (done == 0) {
		if (failed == 0) {
			user_error ("No drawings to translate");
		} else {
			user_error ("Could not translate any drawings");
		}
	} else if (failed == 0) {
		user_inform ("Translated all %u drawings, failed on none", done);
	} else {
		user_inform ("Translated %u drawings, failed on %u", done, failed);
	}
	return True;
}

/******************************************************************/
/*  This is the main interface, does parameter checking etc  */
/******************************************************************/

extern Boolean Map_translate (source, new)
	Map source, new;
{
	Entity source_class, new_class;

	PROC ("Map_translate");
	if ( ! source) {
		user_error ("No source map selected for translation");
	} else if ( ! new) {
		user_error ("No destination map selected for translation");
	} else if ( ! source->entity) {
		user_error ("Source map must be loaded for translation");
	} else if ( ! new->entity) {
		user_error ("Destination map must be loaded for translation");
	} else if (sizeOf_drawingList (Map_elements (new)) > 0) {
		user_error ("Destination map must not have any drawings in it");
	} else {
		source_class = Entity_classOf(source->entity);
		new_class    = Entity_classOf(new->entity);
		if (new_class == source_class)
			user_error ("Source and destination maps already have same class [%s]",
				Entity_title (new_class));
		else
			return do_translate (source, new, source_class, new_class);
	}
	return False;
}

/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/translate.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/

