/*****************************************************************************************************/
/*****************************************************************************************************/
/*  $Header$  */
/*****************************************************************************************************/
/*  Provides an interface between the cmusnmp library and the snmp user modules */
/*****************************************************************************************************/

#include "general/General.h"
#include <sys/types.h>
#include <netinet/in.h>
#include "query/SnmpLib.h"


/*****************************************************************************************/
/*  Globals  */
/*****************************************************************************************/

static Boolean Pinging;

Oid	ifInOctets,
	ifOutOctets,
	ifOperStatus,
	ifSpeed,
	sysUpTime,
	ifLastChange,
	ifNumber;





/*****************************************************************************************/

/*****************************************************************************************/

#ifdef CMU_SNMP

extern Boolean text2sysOid (var, oid)
	char * var;
	Oid *oid;
{
	oid->length = MAX_NAME_LEN;
	if (!read_objid(var, oid->name, &(oid->length))) {
		user_error ("Invalid object identifier: [%s]", var);
		oid->length = 0;
		return False;
	}
	return True;
}

extern Boolean	init_snmplib ()
{
	init_mib();
	(void) text2sysOid ("system.sysUpTime.0",    &sysUpTime);
	(void) text2sysOid ("interfaces.ifNumber.0", &ifNumber);
	(void) text2sysOid ("interfaces.ifTable.ifEntry.ifOperStatus.0", &ifOperStatus);
	(void) text2sysOid ("interfaces.ifTable.ifEntry.ifInOctets.0", &ifInOctets);
	(void) text2sysOid ("interfaces.ifTable.ifEntry.ifOutOctets.0", &ifOutOctets);
	(void) text2sysOid ("interfaces.ifTable.ifEntry.ifSpeed.0", &ifSpeed);
	(void) text2sysOid ("interfaces.ifTable.ifEntry.ifLastChange.0", &ifLastChange);
	Pinging = True;
	init_snmp_trap ();
	return True;
}

extern void	finish_snmplib ()
{
	Pinging = False;
}

extern Boolean snmplib_enabled ()
{
	return Pinging;
}


extern char *
uptime_string(timeticks, buf)
    register u_long timeticks;
    char *buf;
{
    int	seconds, minutes, hours, days;

    timeticks /= 100;
    days = timeticks / (60 * 60 * 24);
    timeticks %= (60 * 60 * 24);

    hours = timeticks / (60 * 60);
    timeticks %= (60 * 60);

    minutes = timeticks / 60;
    seconds = timeticks % 60;

    if (days == 0){
	(void) sprintf(buf, "%d:%02d:%02d", hours, minutes, seconds);
    } else if (days == 1) {
	(void) sprintf(buf, "%d day, %d:%02d:%02d", days, hours, minutes, seconds);
    } else {
	(void) sprintf(buf, "%d days, %d:%02d:%02d", days, hours, minutes, seconds);
    }
    return buf;
}

/******************************************************************/
/*  Snmp Traps  */
/******************************************************************/



/******************************************************************/
/*  Not used !!! */
/******************************************************************/

static oid	objid_sysDescr[] = {1, 3, 6, 1, 2, 1, 1, 1, 0};
static int	length_sysDescr = sizeof(objid_sysDescr)/sizeof(oid);
static oid	objid_sysUpTime[] = {1, 3, 6, 1, 2, 1, 1, 3, 0};
static int	length_sysUpTime = sizeof(objid_sysUpTime)/sizeof(oid);
static oid	objid_ifOperStatus[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 8};
static int	length_ifOperStatus = sizeof(objid_ifOperStatus)/sizeof(oid);
static oid	objid_ifInUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 11};
static int	length_ifInUCastPkts = sizeof(objid_ifInUCastPkts)/sizeof(oid);
static oid	objid_ifInNUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 12};
static int	length_ifInNUCastPkts = sizeof(objid_ifInNUCastPkts)/sizeof(oid);
static oid	objid_ifOutUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 17};
static int	length_ifOutUCastPkts = sizeof(objid_ifOutUCastPkts)/sizeof(oid);
static oid	objid_ifOutNUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 18};
static int	length_ifOutNUCastPkts = sizeof(objid_ifOutNUCastPkts)/sizeof(oid);
static oid	objid_ipInReceives[] = {1, 3, 6, 1, 2, 1, 4, 3, 0};
static int	length_ipInReceives = sizeof(objid_ipInReceives)/sizeof(oid);
static oid	objid_ipOutRequests[] = {1, 3, 6, 1, 2, 1, 4, 10, 0};
static int	length_ipOutRequests = sizeof(objid_ipOutRequests)/sizeof(oid);



/******************************************************************/
#endif  /* CMU_SNMP */

/*****************************************************************************************************/
/*****************************************************************************************************/
/*  $Header$  */
/*****************************************************************************************************/
/*****************************************************************************************************/

