/*****************************************************************************************/
/*****************************************************************************************/
/*  $Header$  */
/*****************************************************************************************/
/*****************************************************************************************/

#include "tcp.h"
#include "general/Gather.h"

#include <signal.h>
#include <sys/errno.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>

#ifndef FD_SETSIZE	/* for 4.2BSD */
#define FD_SETSIZE	(sizeof(fd_set) * 8)
#define NFDBITS		32		/* # of bits in fds_bits[0] */
#define FD_SET(n, p)	((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
#define FD_CLR(n, p)	((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
#define FD_ISSET(n, p)	((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
#define FD_ZERO(p)	bzero((char *)(p), sizeof(*(p)))
#endif

/***************************************************************************/
/*  TCP                                                                    */
/***************************************************************************/

struct tcp_query {
	char * host;
	int port;
};
typedef struct tcp_query * TcpQuery;
#define TCP(m)  ((TcpQuery)  SUBCLASSVALUE(m))

static QueryClass QP = NULL;

/****************************************************************************/
/* Ping                                                                     */
/* !!! The connect may take some time, it is a good idea to make the socket */
/* !!! non-blocking and then select for write.                              */
/****************************************************************************/

static Boolean request (query)
	Query query;
{
	Entity mob = query->entity;
	VStatus vstatus = V_STATUS(mob);
	String host = TCP(query)->host;
	int    port = TCP(query)->port;
	int	sock;
	extern int errno;
	struct	sockaddr_in server;
	struct	hostent *hp;
	Boolean result;
	int status;

	if (!QP->enabled)
		return False;

	bzero(&server, sizeof(server));
	server.sin_family = AF_INET;
	server.sin_port = htons(port);

	if (isdigit(host[0]) &&
		(hp = monet_gethostbyaddr (host, MonetStrlen (host), AF_INET)) == NULL) {
			SetReachStatusMask (vstatus, status_invalid);
	} else if ((hp = monet_gethostbyname(host)) == NULL){
		SetReachStatusMask (vstatus, status_invalid);
	} else {
		bcopy(hp->h_addr, &server.sin_addr, hp->h_length);
		if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
			SetReachStatusMask (vstatus, status_unknown);
		} else {
			if (connect(sock, &server, sizeof(server)) < 0) {
				if (errno == ENETUNREACH || errno == ETIMEDOUT) {
					SetReachStatusMask (vstatus, status_unreachable);
				} else { /* !!! */
					SetReachStatusMask (vstatus, status_unknown);
				}
			} else {
				SetReachStatusMask (vstatus, status_reachable);
			}
			close(sock);
		}
	}
	return True;
}

/********************************************************************/
/*  Free  */
/********************************************************************/

static void free (mob)
	Query mob;
{
	MonetFree (TCP(mob)->host);
	MonetFree (TCP(mob));
}                           

/********************************************************************/
/*  Find  */
/********************************************************************/

static  Query find (text)
	String text;
{
	int cc;
	Query o;
	char host[512];
	unsigned int port;

	cc = sscanf (text, "%[^ \t\n+]+%u", host, &port);
	if (cc != 2)
		return NULL;

	FORINSTANCES (o, QP)
		if (TCP(o)->port == port && EQS(TCP(o)->host, host))
			return o;

	return NULL;
}

/********************************************************************/
/*  Create  */
/********************************************************************/

static  SubValue create (query, text)
	String text;
	Query query;
{
	int cc;
	char host[512];
	int port;

	cc = sscanf (text, "%[^ \t\n\+]+%u", host, &port);
	if (cc == 2) {
		struct	hostent *hp;
		TcpQuery new;
		new = MonetNew (struct tcp_query);
		new -> port      = port;
		if (isdigit(host[0]))
			hp = monet_gethostbyaddr (host, MonetStrlen (host), AF_INET);
		else
			hp = monet_gethostbyname(host);

		if (hp == NULL)
			new -> host = MonetNewString (host);
		else
			new -> host = MonetNewString (hp->h_name);
		return (SubValue) new;
	} else {
		user_error ("tcp:<host>+<port> seems to be incorrect [%s]", text);
 		return NULL;
	}
	/* NOTREACHED */
}

/********************************************************************/
/*  Convert to text */
/********************************************************************/

static char * as_text (mob)
	Query mob;
{
	static char text [1024]; /* !!! */
	(void) sprintf (text, "tcp:%s+%u", TCP(mob)->host, TCP(mob)->port);
	return text;
}

/*****************************************************************************************/
/* Init and Finish */
/*****************************************************************************************/

static void finish ()
{
}

extern QueryClass init_tcp ()
{
	NEW_QUERYCLASS (QP, "tcp");
	return QP;
}

/*****************************************************************************************/
/*****************************************************************************************/
/* $Header$ */
/*****************************************************************************************/
/*****************************************************************************************/

