/*
** Copyright (c) 1990 David E. Smyth
**
** Redistribution and use in source and binary forms are permitted
** provided that the above copyright notice and this paragraph are
** duplicated in all such forms and that any documentation, advertising
** materials, and other materials related to such distribution and use
** acknowledge that the software was developed by David E. Smyth.  The
** name of David E. Smyth may not be used to endorse or promote products
** derived from this software without specific prior written permission.
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*/

/******************************************************************************
**
** SCCS_data: @(#)Mri.c 1.1 ( 19 Nov 90 )
**
** Description:	This file contains main() for a Motif Resource Interpreter
**		which allows prototype interfaces to be built from
**		resource files.  The Widget Creation library is used.
**
**		Besides the Motif widgets, Mri also knows about Table
**		widgets, simply because they are so dang useful!
**
** Notes:	This program uses the Xrm (X resource management) database
**		for widget tree definition and management.  This program
**		is dependent on the Motif widget set only because the
**		Motif classes and constructors are registered, which
**		causes the Motif libs to be linked in.  Someday I'll
**		get a shared lib version of Motif and the Athena widgets,
**		and even the OpenLook widget set, and then there will
**		be no reason that widgets could not be mixed and matched.
**		Doing that without shared libs makes for a HUGE executable.
**
******************************************************************************/

/******************************************************************************
**   Include_files.
******************************************************************************/

#include <Xm/Xm.h>
#include <ctype.h>
#include <Wc/WcCreate.h>

#include <Xmp/Table.h>

/******************************************************************************
**  Private Functions
******************************************************************************/

extern void MriRegisterMotif();

static void RegisterTable ( app )
    XtAppContext app;
{
#define RCN( name, class ) WcRegisterClassName ( app, name, class );
#define RCP( name, class ) WcRegisterClassPtr  ( app, name, class );

    /* -- register widget classes */
    RCN( "Table",                       tableWidgetClass                );
    RCP( "tableWidgetClass",            tableWidgetClass                );
}

/******************************************************************************
*   MAIN function
******************************************************************************/

main ( argc, argv )
    int argc;
    char* argv[];
{   
    char*        appClass;
    XtAppContext app;
    Widget       appShell;

    appClass = (char*) XtMalloc ( strlen ( argv[0] ) + 1 );
    strcpy (appClass, argv[0]);
    /* initialize first letter to make class, or first two if
    ** first is already capitalized, or don't worry about it.
    */
    if (islower(appClass[0]))
	appClass[0] = toupper(appClass[0]);
    else if (islower(appClass[1]))
        appClass[1] = toupper(appClass[1]);
    
    /*  -- Intialize Toolkit creating the application shell */
    appShell = XtInitialize ( 
	argv[0], appClass,		/* app name and class */
	NULL, 0, 			/* description of cmd line options */
	&argc, argv 
    );
    app = XtWidgetToApplicationContext(appShell);

    /*  -- Register all application specific callbacks and widget classes */
    RegisterTable ( app );

    /*  -- Register all Motif classes and constructors */
    MriRegisterMotif ( app );

    /*  -- Create widget tree below toplevel shell using Xrm database */
    WcWidgetCreation ( appShell );

    /*  -- Realize the widget tree and enter the main application loop */
    XtRealizeWidget ( appShell );
    XtMainLoop ( );
}
