
/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*  Rubber band routines  */
/*  This is a first attempt to generalise a set of routines that were        */
/*  originally embedded in the code for 'xmonet'. Hence their rather strange */
/*  structure.                                                               */
/*****************************************************************************/

#include "general/General.h"
#include "xmonet/Widget.h"

GC XMonetRubberGC;

#define DRAW(x,y,width,height) XDrawRectangle(display, XtWindow(widget), \
			XMonetRubberGC, x, y, width, height);

/*****************************************************************************/

/*****************************************************************************/

static void adjust_forGrid (grid_x, grid_y, x,y, newx,newy)
	Cardinal grid_x, grid_y;
	Position x,y, *newx, *newy;
{
	*newx = (x >> grid_x) << grid_x;
	*newy = (y >> grid_y) << grid_y;
}

/*****************************************************************************/

/*****************************************************************************/

extern Boolean Xio_rubberband_drag (widget, grid_x, grid_y, drag, done, event)
	Widget widget;
	XioDragCallback drag, done;
	Cardinal grid_x, grid_y;
	XEvent event;
{
	Position x, y, px, py;

/*	SELECTINPUT(window); */

	px = event.xbutton.x;
	py = event.xbutton.y;
	x = px;
	y = py;	
	drag (px, py, x,y);
	do {
		XNextEvent (display, &event);
		/* !!! Does no bound checking or anything */
		if (event.type == MotionNotify) {
			drag (px, py, x,y);
			x = event.xbutton.x;
			y = event.xbutton.y;
			adjust_forGrid (grid_x, grid_y, x, y, &x, &y);
			drag (px, py, x,y);
		} else if (event.type == LeaveNotify) {
			user_error ("Sorry, can't drag something out of a window");
			break;
		}
	} while (event.type != ButtonRelease);
	drag (px, py, x,y);

	if (px==x && py == y)
		return False;
	done (px, py, x,y);
	return True;
}



extern Boolean Xio_rubberband_drag_rectangle (widget, grid_x, grid_y, xloc, yloc, w, h, event)
	Widget widget;
	Position *xloc, *yloc;
	Cardinal grid_x, grid_y;
	Dimension w, h;
	XEvent event;
{
	Position x, y;

/*	SELECTINPUT(window) */

	x = *xloc;
	y = *yloc;

	DRAW(x, y, w, h);
	do {
		XNextEvent (display, &event);
		/* !!! Does no bound checking or anything */
		if (event.type == MotionNotify) {
			DRAW(x, y, w, h);
			x = event.xbutton.x;
			y = event.xbutton.y;
			adjust_forGrid (grid_x, grid_y, (Position) x - w/2, (Position) y - h/2, &x, &y);
			DRAW(x, y, w, h);
		} else if (event.type == LeaveNotify) {
			user_error ("Sorry, can't drag something out of a window");
			break;
		}
	} while (event.type != ButtonRelease);
	DRAW(x, y, w, h);
	if (*xloc == x && *yloc == y)
		return False;
	*xloc = x;
	*yloc = y;
	return True;
}

static draw_outlinebox (widget, xl,yl, xr,yr)
	Widget widget;
	Position xl,yl, xr,yr;
{
	Position x,y;
	Dimension w,h;

	if (xl < xr)
		x = xl;
	else
		x = xr;
	if (yl < yr)
		y = yl;
	else
		y = yr;
 	w = abs (xr-xl);
 	h = abs (yr-yl);

	DRAW(x, y, w, h);
}

extern Boolean Xio_rubberband_makebox (widget, xloc, yloc, width, height, event)
	Widget widget;
	Position *xloc, *yloc;
	Dimension *width, *height;
	XEvent event;
{
	/* The bounds of the box */
	Position x, y;
	Dimension w,h;

	/* The cursor Position */
	Position cx, cy;
	Position fixedx, fixedy;

	x = *xloc;
	y = *yloc;
	w = *width;
	h = *height;
	cx = event.xbutton.x;
	cy = event.xbutton.y;

	if (cx < x + w /2) {
		fixedx = x + w;
	} else {
		fixedx = x;
	}
	if (cy < y + h /2) {
		fixedy = y + h;
	} else {
		fixedy = y;
	}

	draw_outlinebox (widget, cx,cy, fixedx,fixedy);
	do {
		XNextEvent (display, &event);
		/* !!! Does no bound checking or anything */
		if (event.type == MotionNotify) {
			draw_outlinebox (widget, cx,cy, fixedx,fixedy);
			cx = event.xbutton.x;
			cy = event.xbutton.y;
			draw_outlinebox (widget, cx,cy, fixedx,fixedy);
		} else if (event.type == LeaveNotify) {
			user_error ("Sorry, can't resize something out of a window");
			break;
		}
	} while (event.type != ButtonRelease);

	draw_outlinebox (widget, cx,cy, fixedx,fixedy);  


	if (cx < fixedx) {
		x = cx;
		w = fixedx - cx;
	} else {
		x = fixedx;
		w = cx - fixedx;
	}
	if (cy < fixedy) {
		y = cy;
		h = fixedy - cy;
	} else {
		y = fixedy;
		h = cy - fixedy;
	}
	if (*xloc == x && *yloc == y && *width == w && *height == h)
		return False;
	*xloc = x;
	*yloc = y;
	*width = w;
	*height = h;
	return True;
}

/*****************************************************************************/

/*****************************************************************************/




/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/
