/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

#include "general/General.h"
#include "xmonet/Widget.h"
#include "general/Help.h"
#include "general/Channel.h"

extern GC XMonetRubberGC;

/*****************************************************************************/
/*  Handling listening to fds  */
/*****************************************************************************/

struct input_fid {
	int fid;
	XtInputId xtid;
	InputFid next;
	FidHandler handler;
	caddr_t data;
};

static InputFid InputFidList = NULL;

/*****************************************************************************/
/*  Memory allocation  */
/*****************************************************************************/

extern char * MonetRealloc (s, n)
	char * s;
	int n;
{
	return XtRealloc (s, n);
}

extern char * MonetMalloc (i)
	int i;
{
	return XtMalloc (i);
}

extern void MonetFree (i)
	char * i;
{
	XtFree (i);
}


/*****************************************************************************/
/*  Return position of widget  */
/*****************************************************************************/

extern void Widget_get_dimensions (w, x,y,width,height)
	Widget w;
	Position *x,*y;
	Dimension *width, *height;
{
	/*  If not realised then core values are rubbish  !!! */
	if (w != (Widget) 0) {
		*x = w->core.x;
		*y = w->core.y;
		*width = w->core.width;
		*height = w->core.height;
	} else {
		*x = 0;
		*y = 0;
		*width = 0;
		*height = 0;
	}
}

/***************************************************************************/
/*  Handle list of inputs waiting for select  */
/***************************************************************************/

static void Handler (closure, source, id)
    XtPointer closure;
    int*      source;
    XtInputId* id;
{
	InputFid call = (InputFid) closure;
	call->handler (call->fid,call->data);
}

extern InputFid create_input_fid (fid, type, handler, data)
	int fid;
	FidHandler handler;
	FidType type;
	caddr_t data;
{
	InputFid new;
	struct  call;

	new = MonetNew (struct input_fid);
	new -> fid = fid;
	new -> next = InputFidList;
	new -> handler = handler;
	new -> data = data;
	InputFidList = new;

	if (type == fid_read)
		new -> xtid = XtAppAddInput (app_con, fid, XtInputReadMask, Handler, new);
	else if (type == fid_write)
		new -> xtid = XtAppAddInput (app_con, fid, XtInputWriteMask, Handler, new);
	else
		sys_error ("Unknown Fid Type [%d]", type);

	return new;
}

extern void free_input_fid (fid)
	int fid;
{
	InputFid old = NULL, i;

	FORALL(i, InputFidList, next)
		if (i->fid == fid)
			old = i;
	if (old == NULL) {
		sys_error ("input fid not found %d", fid);
		return;
	}
	if (old->xtid != NULL)
		XtRemoveInput (old->xtid);

	if (InputFidList == old)
		InputFidList = old -> next;
	else
		FORALL(i, InputFidList, next)
			if (i->next == old)
				i->next = old->next;
	close (fid);
	MonetFree (old);
}

/*****************************************************************************/
/*  Add a help feature to the menu buttons  */
/*****************************************************************************/

static void set_temporary_help_mode ()
{
	 set_helpMode(help_temporary);
}


static char defaultTranslations[] = {
      	"<Btn2Down>:	givehelp()"
};

static XtTranslations TransTable = NULL;

/*****************************************************************************/
/*  I don't particularly like making these global */
/*****************************************************************************/

Widget          toplevel;
XtAppContext    app_con;
Display         *display;

/******************************************************************************/
/*  General purpose widget routines  */
/******************************************************************************/

extern Widget create_commandWidget (label, parent, callback, data)
	Widget parent;
	String label;
	XtCallbackProc callback;
	XtPointer data;
{
	Widget button;

	button = XtCreateManagedWidget(label, commandWidgetClass, parent, (Arg *) 0, (Cardinal) 0);
	if (callback != NULL)
		XtAddCallback(button, XtNcallback, callback, (XtPointer) data);
	return button;
}


/******************************************************************************/
/*  Dialog Boxes */
/******************************************************************************/

extern Widget create_dialogWidget (label, parent, value)
	Widget parent;
	String label;
	String value;
{
	Widget dialog;
	Arg    args[5];

	XtSetArg(args[0], XtNresizable, False);
	XtSetArg(args[1], XtNvalue, value);
	XtSetArg(args[2], XtNlabel, label);
	dialog = XtCreateManagedWidget(label, dialogWidgetClass, parent, args, (Cardinal) 3);

	return dialog;
}

extern Widget create_dialogButton (label, parent, callback, data)
	Widget parent;
	String label;
	XtCallbackProc callback;
	XtPointer data;
{
	XawDialogAddButton (parent, label, callback, data);
	return NULL;
}

/******************************************************************************/
/*  Dialog box  value*/
/*  If widget is not a dialog box, will ask parent  */
/******************************************************************************/

extern String get_dialogWidgetValue (w)
	Widget w;
{
	for (;w; w = XtParent(w))
		if (XtIsSubclass (w, dialogWidgetClass))
			return XawDialogGetValueString (w);
	user_error ("Neither Widget nor parents is a dialog box");
	return NULL;
}

/******************************************************************************/
/*  Menus  */
/******************************************************************************/

extern Widget create_menu (label, parent)
	Widget parent;
	String label;
{
	Widget button, menu;
	Arg args [1];

	/*  XtSetArg (args[0], XtNmenuName, label); */
	button = XtCreateManagedWidget(label, menuButtonWidgetClass, parent, args, (Cardinal) 0);
	menu = XtCreatePopupShell ("menu", simpleMenuWidgetClass, button, (Arg*) 0, (Cardinal) 0);
	return menu;
}

extern Widget create_menuEntryWidget (label, parent, callback, data)
	Widget parent;
	String label;
	XtCallbackProc callback;
	XtPointer data;
{
	Widget button;
	button = XtCreateManagedWidget(label, smeBSBObjectClass, parent, (Arg *) 0, (Cardinal) 0);
	if (callback != NULL)
		XtAddCallback(button, XtNcallback, callback, (XtPointer) data);
/*	(void) XtAugmentTranslations(button, TransTable); */
	return button;
}


extern Widget create_menuLineWidget (parent)
	Widget parent;
{
	return  XtCreateManagedWidget("line", smeLineObjectClass, parent, (Arg *) 0, (Cardinal) 0);
}



/******************************************************************************/
/*  Popup box  */
/******************************************************************************/


static void PopupWidget (widget, client_data, call_data)
	Widget          widget;
	XtPointer       client_data, call_data;
{
	Widget         popup = (Widget) client_data;
	Arg        args[5];
	Position x,y;

	XtTranslateCoords(XtParent (popup), (Position) 0, (Position) 0, &x, &y);
	XtSetArg(args[0], XtNx, x);
	XtSetArg(args[1], XtNy, y+25);
	XtSetValues(popup, args, 2);

	XtPopup(popup, XtGrabNone);
}

static void PopdownWidget (widget, client_data, call_data)
	Widget          widget;
	XtPointer       client_data, call_data;
{
	Widget         popup = (Widget) client_data;
	Arg        args[4];

	XtPopdown(popup);
}


extern Widget create_popupBox (label, parent)
	String label;
	Widget parent;
{
	Widget box, popup, button;
	Arg args[5];

	button = XtCreateManagedWidget(label, commandWidgetClass, parent, args, ZERO);
	popup = XtCreatePopupShell ("popup", transientShellWidgetClass, button, args, ZERO);
	XtAddCallback(button, XtNcallback, PopupWidget, (XtPointer) popup);
 	box = XtCreateManagedWidget("box", boxWidgetClass, popup, args, ZERO);
	button = XtCreateManagedWidget("Exit", commandWidgetClass, box, args, ZERO);
	XtAddCallback(button, XtNcallback, PopdownWidget, (XtPointer) popup);


	return box;
}

/******************************************************************************/

/******************************************************************************/

extern Widget create_boxWidget (label, parent)
	String label;
	Widget parent;
{
  	return XtCreateManagedWidget(label, boxWidgetClass, parent, (Arg *) 0, (Cardinal) 0);
}


extern Widget create_labelWidget (label, parent)
	String label;
	Widget parent;
{
	return  XtCreateManagedWidget (label, labelWidgetClass, parent, (Arg*) 0, (Cardinal) 0);
}

extern Widget create_textWidget (buffer, size, parent)
	String buffer;
	Cardinal size;
	Widget parent;
{
	Arg args[5];

	XtSetArg (args[0], XtNstring, buffer);
	XtSetArg (args[1], XtNlength, (int) size);
	XtSetArg (args[2], XtNeditType, XttextEdit);
	XtSetArg (args[3], XtNwidth, 200);
	XtSetArg (args[4], XtNtextOptions, editable | scrollOnOverflow | scrollHorizontal);
	return  XtCreateManagedWidget ("text", asciiStringWidgetClass, parent, args, 5);
}

extern void popup_central (parent, popup, grab)
	Widget parent, popup;
	XtGrabKind grab;
{
	Position x,y;
	Arg             args[10];

	if (!XtIsRealized(popup))
		XtRealizeWidget(popup);

	if (!XtIsRealized(parent))
		XtRealizeWidget(parent);

	x = parent->core.x + parent->core.width /2  - popup->core.width /2;
	y = parent->core.y + parent->core.height /2 - popup->core.height /2;

	XtSetArg(args[0], XtNx, x);
	XtSetArg(args[1], XtNy, y);
	XtSetArg(args[2], XtNresizable, False);
	XtSetValues(popup, args, 3);
	XtPopup (popup, grab);
}

/*****************************************************************************/
/*  Utility to write messages to end of text widgets */
/*****************************************************************************/

extern void put_widget (w, add_this)
	Widget w;
	String add_this;
{
	XtTextBlock text;
	static XtTextPosition pos = 0;    /* !!!  This depends on the widget used */
	int length;

	if (w == (Widget) 0)
		return;
	if (XtIsRealized (w) == False)
		return;

	length = MonetStrlen(add_this);

	text.length = length;
	text.ptr = add_this;
	text.firstPos = 0;
	text.format = FMT8BIT;

	XtTextReplace (w, pos, pos, &text);

	pos += text.length;
	XtTextSetInsertionPoint(w, pos);
}

extern void write_widget (w, add_this, a,b,c,d,e,f,g,h,i)
Widget w;
String add_this;
{
#define SIZEMESSAGEBUFFER  2048   /* !!! */
	char Buffer[SIZEMESSAGEBUFFER];
	
	(void) sprintf (Buffer, add_this, a,b,c,d,e,f,g,h,i);
	put_widget (w, Buffer);
}

extern void writeln_widget (w, add_this, a,b,c,d,e,f,g,h,i)
Widget w;
String add_this;
{
	write_widget (w, add_this, a,b,c,d,e,f,g,h,i);
	put_widget (w, " \n");
} 

/****************************************************************************************************/
/*  X11 Messages widget  */
/*  !!! This could be integrated better  */
/*  Core Basic Message Routines */
/****************************************************************************************************/

static Widget Messages_widget = (Widget) 0;
extern void create_messages_form (parent)
Widget parent;
{
	Arg args[6];
	
	PROC ("create_messages_form");

#ifdef X11R4
XtSetArg (args[0], XtNeditType, XawtextEdit);
XtSetArg (args[1], XtNscrollVertical, True);
XtSetArg (args[2], XtNscrollHorizontal, True);
XtSetArg (args[3], XtNheight, 150);
	XtSetArg (args[4], XtNwidth, 600);
	Messages_widget =  XtCreateManagedWidget ("user_messages", asciiTextWidgetClass, parent, args, 5);
#else
	XtSetArg (args[0], XtNeditType, XttextEdit);

	XtSetArg (args[1], XtNtextOptions, scrollVertical | scrollHorizontal | scrollOnOverflow | editable);

	XtSetArg (args[2], XtNheight, 75);   /* !!! Should be 3x Font hieght */
	XtSetArg (args[3], XtNwidth, 400);   /* !!! Should maybe be elsewhere */

		XtSetArg (args[4], XtNlength, 40000);  /* !!! Shouldn't fix this at a value */
	Messages_widget =  XtCreateManagedWidget ("messages", asciiStringWidgetClass, parent, args, 5);
#endif
}

static void xio_user_message (label, a,b,c,d,e,f,g,h,i)
     char * a, * label;
{
	if (Messages_widget) {
		if (label)
			write_widget (Messages_widget, "[%s] ", label);
		writeln_widget (Messages_widget, a,b,c,d,e,f,g,h,i);
	} else {
		if (label)
			printf ("[%s]", label);
		printf (a,b,c,d,e,f,g,h,i);
		printf ("\n");
	}
}

CALLBACK(SetUserMessageWidgetCB)
	String w_name = (String) client_data;

	Widget text_widget;
	String name;
	char  clean_name[1024]; /* !!! */

	if (w_name == NULL) {
		user_error ("Messages text widget name is NULL");
		return;
	}
	WcCleanName (w_name, clean_name);
	if ( !toplevel) {
		sys_error ("Toplevel widget is NULL");
		return;
	}
	text_widget = WcFullNameToWidget (toplevel, clean_name);
	if ( ! text_widget) {
		user_error ("No such text widget [%s]", clean_name);
		return;
	}
	if ( ! XtIsSubclass (text_widget, textWidgetClass)) {
		user_error ("Messages Widget [%s] is not an text widget", clean_name);
		return;
	}
	(void) set_user_inform_routine (xio_user_message);
	Messages_widget = text_widget;
ENDCB

/*****************************************************************************/
/*  !!!  This stuff not used I think  */
/*****************************************************************************/

extern Position XCore_x (w)
	Widget w;
{ return w->core.x; }

extern Position XCore_y (w)
	Widget w;
{ return w->core.y; }

extern Position XCore_w (w)
	Widget w;
{ return w->core.width; }

extern Position XCore_h (w)
	Widget w;
{ return w->core.height; }

/*****************************************************************************/
/* A couple of actions for popping things up */
/*****************************************************************************/

ACTION(PopupCommandPopupACT, 1)
	PopupCommandPopupCB (widget, params[0], False);
ENDACT

ACTION(PopupCommandPopupGrabACT, 1)
	PopupCommandPopupCB (widget, params[0], (XtPointer) True);
ENDACT

ACTION(PopupAtCursorACT, 1)
	PopupAtCursorCB (widget, params[0], NULL);
ENDACT

ACTION(SetReportWidgetACT, 1)
	SetReportWidgetCB(widget, params[0], 0);
ENDACT

/*****************************************************************************/
/* Start X client                                                            */
/* !!! Should probably load fonts and such like at this stage                */
/*****************************************************************************/

extern Boolean XMonet_init (appClass, argc, argv)
	char*        appClass;
	int             argc;
	char          **argv;
{
	int             screen;
	XGCValues       values;
	unsigned int    mask;
	Pixel           background, foreground;
#define NUM_ACTIONS 5
	static XtActionsRec actions[NUM_ACTIONS] = {
	{"PopupCommandPopupACT", PopupCommandPopupACT},
	{"PopupCommandPopupGrabACT", PopupCommandPopupGrabACT},
	{"PopupAtCursorACT", PopupAtCursorACT},
	{"SetReportWidgetACT", SetReportWidgetACT},
	{"givehelp", set_temporary_help_mode}
	};

    /* initialize first letter to make class, or first two if
    ** first is already capitalized, or don't worry about it.
    */
	if (islower(appClass[0]))
		appClass[0] = toupper(appClass[0]);
	else if (islower(appClass[1]))
		appClass[1] = toupper(appClass[1]);
    
    /* With the Athena widgets, we MUST register the Wc
    ** converters first, so WcCvtStringToWidget is invoked
    ** instead of the far less useful XmuCvtStringToWidget.
    */
	WcAddConverters(NULL);

    /*  -- Intialize Toolkit creating the application shell */
	toplevel = XtInitialize ( 
		argv[0], appClass,		/* app name and class */
		NULL, 0, 			/* description of cmd line options */
		&argc, argv 
		);
	if (toplevel == NULL) {
		user_error ("Could not create toplevel shell");
		return False;
	}
	app_con = XtWidgetToApplicationContext(toplevel);
	if (app_con == NULL) {
		user_error ("Could not create application context");
		return False;
	}
	display = XtDisplay (toplevel);

	/*  -- Init drawing routines  */
	init_drawX11(toplevel);

	(void) XtAppAddActions(app_con, (XtActionList) actions, (Cardinal) NUM_ACTIONS);

	/* -- Set output widgets  */
	RCALL("SetUserMessageWidgetCB",SetUserMessageWidgetCB,NULL);
	RCALL("SetReportWidgetCB",SetReportWidgetCB,NULL);

	/* -- General widget routines */
	RCALL("DestroyWidgetCB",	DestroyWidgetCB,	NULL);
	RCALL("DestroyShellCB",		DestroyShellCB,	NULL);

	/* -- Confirm box */
	RCALL("DoConfirmActionCB",	DoConfirmActionCB,	NULL);
	RCALL("RegisterConfirmPopupCB",	RegisterConfirmPopupCB,	NULL);
	RCALL("RegisterConfirmMessageCB",RegisterConfirmMessageCB,	NULL);

	/* -- popup boxes and menus  */
	RCALL("PopupCommandPopupCB",	PopupCommandPopupCB,	False);
	RCALL("PopupCommandPopupGrabCB",PopupCommandPopupCB,	True);
	RCALL("PopupAtCursorCB",	PopupAtCursorCB, 	NULL);
	RCALL("PopupCentralCB",		PopupCentralCB,		NULL);
	RCALL("MonetLoadFileInWidgetCB",MonetLoadFileInWidgetCB, NULL);

	/*  -- Register all Athena widget classes */
	AriRegisterAthena ( app_con );

#ifdef HAS_MOTIF
	/*  -- Register all Motif classes and constructors */
	MriRegisterMotif ( app_con );
#endif /* HAS_MOTIF */

	/**** Set up back and foreground pixels ****/
	/* !!! Not guaranteed to work on colour or greyscale displays */
	screen = DefaultScreen(display);
	background = BlackPixel(display, screen);
	foreground = WhitePixel(display, screen);
	/* !!! Hack */
	if (foreground == 0) {
		foreground = background;
		background = 0;
	}
	/**** Set up the GC's ****/
	mask  = GCForeground | GCBackground | GCLineStyle |
		GCLineWidth | GCFillStyle | GCJoinStyle | GCCapStyle | GCFunction;
	values.function   = GXxor;
	values.foreground = foreground;
	values.background = background;
	values.line_style = LineSolid;
	values.line_width = 1;
	values.cap_style  = CapRound;
	values.fill_style = FillSolid;
	values.join_style = JoinRound;
	XMonetRubberGC = XtGetGC(toplevel, mask,  &values);

	return True;
}

/*****************************************************************************/
/* Top Level Loop                                                    */
/*****************************************************************************/

void XMonet_EventLoop()
{
	XEvent          event;

	PROC ("Loop");
	XtRealizeWidget(toplevel);
	XFlush(XtDisplay(toplevel));	/* this ensures that all requests are
					 * sent */
	/* without this line, linking with cc means */
	/* that the requests are not all sent */


	for (;;) {
		XtAppNextEvent(app_con, &event);
		XtDispatchEvent(&event);
	}
	/* XtAppMainLoop(app_con); */
	/* NOTREACHED */
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

